#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <gtk/gtk.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/encoding.h>
#include <libxml/xmlwriter.h>
#include <unistd.h>

#include "support.h"
#include "xml_work.h"

#define LSB_FILE "lsb_release"
#define RPM_OUTPUT "rpm_output"
#define MY_ENCODING "UTF-8"

void
xml_generate_lsbrelease ()
{
	FILE *fd;
	gchar filename[512] = "\0", xml_filename[512] = "\0";
	gchar distr[256] = "\0", release[64] = "\0", distr_id[256] = "\0", codename[256] = "\0";
	xmlTextWriterPtr writer;
	xmlChar *tmp = NULL;
	gint rc;
	ssize_t read_st;
        size_t len = 0;
        gchar *line = NULL;


	g_sprintf (filename, "%s/%s/%s", g_get_home_dir (), CACHE_DIR, LSB_FILE);
	fd = fopen (filename, "r");
	if (!fd)
	{
		exit(1);
	}

	while ((read_st = getline(&line, &len, fd)) != -1) 
	{
	    if (strstr (line, "Distributor ID:"))
	    {
			g_sprintf (distr_id, "%s", line+16);
			choppy (distr_id, '\0');
	    }

	    if (strstr (line, "Description:"))
	    {
			g_sprintf (distr, "%s", line+13);
			choppy (distr, '\0');
	    }

	    if (strstr (line, "Release:"))
	    {
			g_sprintf (release, "%s", line+9);
			choppy (release, '\0');
	    }

	    if (strstr (line, "Codename:"))
	    {
			g_sprintf (codename, "%s", line+10);
			choppy (codename, '\0');
	    }
	}
	fclose (fd);
	
	g_sprintf (xml_filename, "%s.xml", filename);
	writer = xmlNewTextWriterFilename (xml_filename, 0);

	rc = xmlTextWriterStartDocument (writer, NULL, MY_ENCODING, NULL);
	if (rc < 0)
	{
		exit(1);
	}

	rc = xmlTextWriterStartElement (writer, BAD_CAST "lsb_release");
	if (rc < 0)
	{
		exit(1);
	}

	tmp = ConvertInput (distr_id, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "distributor_id", tmp);
	if (rc < 0)
	{
		exit(1);
	}

	tmp = ConvertInput (distr, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "description", tmp);
	if (rc < 0)
	{
		exit(1);
	}

	tmp = ConvertInput (release, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "release", tmp);
	if (rc < 0)
	{
		exit(1);
	}

	tmp = ConvertInput (codename, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "codename", tmp);
	if (rc < 0)
	{
		exit(1);
	}

	rc = xmlTextWriterEndElement(writer);
	rc = xmlTextWriterEndDocument(writer);

	xmlFreeTextWriter(writer);
	xmlFree (tmp);
	g_free (line);
}

void
xml_generate_protocols ()
{
	FILE *fd;
	gchar xml_filename[512] = "\0";
	gchar _protocol_version[64], _client_version[128];
	gchar protocol_name[128], client_name[128];
	xmlTextWriterPtr writer;
	xmlChar *tmp;
	gint rc;

	fd = fopen (PROTOCOLS_VERSION, "r");		
	if (!fd)
	{
		exit (1);
	}

	rc = fscanf (fd, "%s %s", protocol_name, _protocol_version);
	fclose (fd);

	fd = fopen (CLIENT_VERSION, "r");		
	if (!fd)
	{
		g_printf ("Error! Can not open file: %s\n", CLIENT_VERSION);
		exit (1);
	}

	rc = fscanf (fd, "%s %s", client_name, _client_version);
	fclose (fd);

	g_sprintf (xml_filename, "%s/%s/client.xml", g_get_home_dir (), CACHE_DIR);
	writer = xmlNewTextWriterFilename (xml_filename, 0);

	rc = xmlTextWriterStartDocument (writer, NULL, MY_ENCODING, NULL);
	if (rc < 0)
	{

		exit(1);
	}

	rc = xmlTextWriterStartElement (writer, BAD_CAST "client_info");
	if (rc < 0)
	{

		exit(1);
	}

	rc = xmlTextWriterStartElement (writer, BAD_CAST "client");
	if (rc < 0)
	{

		exit(1);
	}


	tmp = ConvertInput (client_name, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "name", tmp);
	if (rc < 0)
	{

		exit(1);
	}

	tmp = ConvertInput (_client_version, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "version", tmp);
	if (rc < 0)
	{

		exit(1);
	}

	rc = xmlTextWriterEndElement(writer);

	rc = xmlTextWriterStartElement (writer, BAD_CAST "protocol");
	if (rc < 0)
	{

		exit(1);
	}

	tmp = ConvertInput (protocol_name, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "name", tmp);
	if (rc < 0)
	{

		exit(1);
	}

	tmp = ConvertInput (_protocol_version, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "version", tmp);
	if (rc < 0)
	{

		exit(1);
	}

	rc = xmlTextWriterEndElement(writer);

	rc = xmlTextWriterEndElement(writer);
	rc = xmlTextWriterEndDocument(writer);

	xmlFreeTextWriter(writer);
	xmlFree (tmp);
}

void
xml_generate_packages ()
{
	FILE *fd;
	gchar filename[512] = "\0", xml_filename[512] = "\0";
	gchar rpm_name[256] = "\0", rpm_version[64] = "\0", rpm_summary[512];
	xmlTextWriterPtr writer;
	xmlChar *tmp;
	gint rc;
	ssize_t read_st;
        size_t len = 0;
        gchar *line = NULL;

	g_sprintf (filename, "%s/%s/%s", g_get_home_dir (), CACHE_DIR, RPM_OUTPUT);
	fd = fopen (filename, "r");		
	if (!fd)
	{
		exit (1);
	}	

	g_sprintf (xml_filename, "%s/%s/packages.xml", g_get_home_dir (), CACHE_DIR);
	writer = xmlNewTextWriterFilename (xml_filename, 0);

	rc = xmlTextWriterStartDocument (writer, NULL, MY_ENCODING, NULL);
	if (rc < 0)
	{
		exit(1);
	}

	rc = xmlTextWriterStartElement (writer, BAD_CAST "packages");
	if (rc < 0)
	{
		exit(1);
	}	
	
	while ((read_st = getline(&line, &len, fd)) != -1) 
	{
		sscanf (line, "%1024[^$]$%1024[^$]$%1024[^\n]", rpm_name, rpm_version, rpm_summary);
		rc = xmlTextWriterStartElement(writer, BAD_CAST "package");
		if (rc < 0)
		{
	
			exit(1);
		}	

		tmp = ConvertInput (rpm_name, MY_ENCODING);
		rc = xmlTextWriterWriteElement(writer, BAD_CAST "name", tmp);
		if (rc < 0)
		{
			return;
		}

		tmp = ConvertInput (rpm_version, MY_ENCODING);
		rc = xmlTextWriterWriteElement(writer, BAD_CAST "version", tmp);
		if (rc < 0)
		{
			return;
		}

		tmp = ConvertInput (rpm_summary, MY_ENCODING);
		rc = xmlTextWriterWriteElement(writer, BAD_CAST "summary", tmp);

		if (rc < 0)
		{
			return;
		}
		rc = xmlTextWriterEndElement(writer);
	}

	free (line);
	rc = xmlTextWriterEndElement(writer);
	rc = xmlTextWriterEndDocument(writer);

	xmlFreeTextWriter(writer);
	xmlFree (tmp);
}


void
xml_generators ()
{
	xml_generate_lsbrelease ();
	xml_generate_protocols ();
	xml_generate_packages ();
}
