#include "improver_sqlite.h"

#define SQL_LOGPATH "/usr/share/improver_client/sqlite.log"

FILE* fLogOpen;
int flagDebugMode;


enum log {
  LOG_NO, // лога нет
  LOG_STDOUT = 1, // в stdout
  LOG_FILE   // в файл
};

#define NameFunction "close_db"
static int 
close_db		(sqlite3 *db) 
{
	int rc = 0;

	if (!db) 
	{
    		WriteLog(NameFunction, "Error: Function parametr is empty!\n");
	        return -1;
	}

        if (db) 
        {
    		rc = sqlite3_close(db);
		if (rc == SQLITE_OK)
		{
        		db = NULL;
        	}
	        else
	        {
    			WriteLog(NameFunction, "Error: Can't close database: %s!\n", sqlite3_errmsg(db));
    		}
	}

        if ((flagDebugMode == LOG_FILE) && (!fLogOpen))
        {
    		fclose(fLogOpen);
    	}
        return rc;
}

#undef NameFunction


int 
exec_request_db (sqlite3 **db, const char* request) 
{
	char *zErrMsg = 0;
	int rc;

	if (!(*db) || !request) 
	{
	        WriteLog("exec_request_db:	", "Error: Function parametr is empty!\n");
    		return -1;
	}

        rc = sqlite3_exec(*db, request, NULL, 0, &zErrMsg);
	switch (rc) 
	{
    		case SQLITE_OK:
        	break;
	        case SQLITE_ABORT:
    		        WriteLog("exec_request_db:	", "SQLITE_ABORT. SQL error: %s!\n", zErrMsg);
		        sqlite3_free(zErrMsg);
                break;
	        case SQLITE_BUSY:
        		WriteLog("exec_request_db:	", "SQLITE_BUSY. SQL error: %s!\n", zErrMsg);
	                sqlite3_free(zErrMsg);
                break;
	        default:
        		WriteLog("exec_request_db:	", "SQLITE error: %s %s!\n", zErrMsg, request);
			sqlite3_free(zErrMsg);
                break;
    }
    return rc;
}


#define NameFunction "prepare_requst"
static int 
prepare_requst			(sqlite3 **db, 
				 const char* request, 
				 sqlite3_stmt** pStmt) 
{
        int rc;

        if (!(*db) || !request) 
	{
	        WriteLog(NameFunction, "Error: Function parametr is empty!\n");
	        return -1;
	}

        rc = sqlite3_prepare(*db, request, strlen(request), pStmt, NULL);
	if (rc != SQLITE_OK) 
	{
    		WriteLog(NameFunction, "Sql error: %s!\n", sqlite3_errmsg(*db));
        	sqlite3_finalize(*pStmt);
		close_db(*db);
		return -1;
        }
    return rc;
}
#undef NameFunction


int
impr_sql_init (const char *dbname, sqlite3 **db)
{
	int rc;
	fLogOpen = fopen (SQL_LOGPATH, "a+");

	if (!fLogOpen)
	{
		printf ("Cant open log file for sql %s\n", SQL_LOGPATH);
		return -1;
	}

	flagDebugMode = LOG_FILE;

        if (*db == NULL) 
	{
	        rc = sqlite3_open(dbname, db);

    		if (rc) 
		{
            		WriteLog("exec_request_db:	", "Error: Can't open database!");
	        }
	}

	return 0;
}


/* User operations */

#define NameFunction "impr_sql_user_add\t"
int
impr_sql_user_add		(int uid, 
				 char *hwid, 
				 char *name, 
				 char *lastname,
				 char *email)
{
	char request[1024];
	sqlite3_stmt* pStmt;
	int rc;

	sprintf (request, "INSERT INTO user VALUES (?, ?, ?, ?, ?)");
	
	rc = prepare_requst(&db, request, &pStmt);
        if (rc != SQLITE_OK)
	{
	    	return -1;
	}

	if (sqlite3_bind_int(pStmt, 1, (int)(uid)) != SQLITE_OK) 
	{
	        WriteLog(NameFunction, "Error: Couldn't bind parametr #1 !\n");
    		return -1;
	}

	if (sqlite3_bind_text(pStmt, 2, hwid, -1, SQLITE_TRANSIENT) != SQLITE_OK)
	{
	        WriteLog(NameFunction, "Error: Couldn't bind parametr #2 !\n");
    		return -1;
        }

	if (sqlite3_bind_text(pStmt, 3, name, -1, SQLITE_TRANSIENT) != SQLITE_OK)
	{
	        WriteLog(NameFunction, "Error: Couldn't bind parametr #3 !\n");
	        return -1;
        }

	if (sqlite3_bind_text(pStmt, 4, lastname, -1, SQLITE_TRANSIENT) != SQLITE_OK)
	{
	        WriteLog(NameFunction, "Error: Couldn't bind parametr #4 !\n");
	        return -1;
	}

	if (sqlite3_bind_text(pStmt, 5, email, -1, SQLITE_TRANSIENT) != SQLITE_OK)
	{
	        WriteLog(NameFunction, "Error: Couldn't bind parametr #5 !\n");
	        return -1;
        }

	rc = sqlite3_step(pStmt);
        if (rc != SQLITE_DONE) 
        {
    		WriteLog(NameFunction,  "Error: %s!\n", sqlite3_errmsg(db));
	        return rc;
	}

        rc = sqlite3_last_insert_rowid(db);
	if (rc < 0)
	{
    		WriteLog(NameFunction,  "Error: It's not insert row!\n");
	}
        sqlite3_reset(pStmt);
	sqlite3_finalize(pStmt);

	return 0;
}
#undef NameFunction


#define NameFunction "impr_sql_user_init\t"
int
impr_sql_user_init 		(int uid, 
				 user_st *user, 
				 char *name, 
				 char *lastname, 
				 char *email, 
				 char *hwid)
{
	char request[512], *value = NULL;
	sqlite3_stmt* pStmt;
	int rc, coln, size, res = 0, f;

	sprintf (request, "SELECT hwid, name, lastname, email FROM user WHERE uid = %d;", uid);
//	printf ("[%s]\n",request);
	rc = prepare_requst(&db, request, &pStmt);
        if (rc != SQLITE_OK)
	{
		return -1;
	}

	bzero (user->name, 256);
	bzero (user->second, 256);
	bzero (user->hw, 64);
	bzero (user->email, 256);

	f = 0;
        while ((rc = sqlite3_step(pStmt)) == SQLITE_ROW) 
	{
		f = 1;
		coln = sqlite3_data_count(pStmt);
    		if (coln == 4) 
		{
			size = sqlite3_column_bytes(pStmt, 0);
	        	value = malloc(size);
    			if (!value) 
			{
		                WriteLog(NameFunction, "Error: Don't malloc memory!\n");
            			return -1;
	        	}

	    		bzero (value, size);
    			memcpy(value, (void *)sqlite3_column_text(pStmt, 0), size);
			value[size] = '\0';			
			sprintf (user->hw, "%s", value);
			free (value);

        		size = sqlite3_column_bytes(pStmt, 3);
	        	value = malloc(size+1);
    			if (!value) 
			{
    				WriteLog(NameFunction, "Error: Don't malloc memory!\n");
	                	return -1;
    			}
	
	    		bzero (value, size);
    			memcpy(value, (void *)sqlite3_column_text(pStmt, 3), size);
			value[size] = '\0';
			sprintf (user->email, "%s", value);
			free (value);

        		size = sqlite3_column_bytes(pStmt, 1);
	        	value = malloc(size);
    			if (!value) 
			{
	            		WriteLog(NameFunction, "Error: Don't malloc memory!\n");
	                	return -1;
	        	}

			bzero (value, size);
    			memcpy(value, (void *)sqlite3_column_text(pStmt, 1), size);
			value[size] = '\0';
			sprintf (user->name, "%s", value);
			free (value);

	        	size = sqlite3_column_bytes(pStmt, 2);
    			value = malloc(size);
        		if (!value) 
			{
	            		WriteLog(NameFunction, "Error: Don't malloc memory!\n");
	            		return -1;
	        	}

			bzero (value, size);
        		memcpy(value, (void *)sqlite3_column_text(pStmt, 2), size);
	    		value[size] = '\0';
			sprintf (user->second, "%s", value);
			free (value);
			res = 1;
    	        }
	}

	sqlite3_reset(pStmt);
	sqlite3_finalize(pStmt);


	if (!f)
	{
//    		printf ("%s %s %s %s\n", hwid, name, lastname, email);
    		impr_sql_user_add (uid, hwid, name, lastname, email);
    		sprintf (user->name, "%s", name);
    		sprintf (user->second, "%s", lastname);
    		sprintf (user->email, "%s", email);
    		sprintf (user->hw, "%s", hwid);
	}
	return 1;
}
#undef NameFunction

int
impr_sql_user_change_pass	(int uid, char *oldpass, char *newpass)
{
	return 0;
}


int 
impr_sql_user_find			(int uid)
{
	return 0;
}

/* Messages operations */
#define NameFunction "impr_sql_message_add\t"
int 
impr_sql_message_add		(int uid, char *text, char *date)
{
	time_t result;
	char request[512], time_str[128];;
	sqlite3_stmt* pStmt;
	int rc;

	result = time(NULL);
	strftime (time_str, 128, "%FT%T", localtime(&result));
	sprintf (request, "INSERT INTO message VALUES (?, ?, ?, ?, NULL)");

	rc = prepare_requst(&db, request, &pStmt);
        if (rc != SQLITE_OK)
	{
		return -1;
	}

	if (sqlite3_bind_text(pStmt, 1, time_str, -1, SQLITE_TRANSIENT) != SQLITE_OK)
	{
	        WriteLog(NameFunction, "Error: Couldn't bind parametr #2 !\n");
    		return -1;
        }


	if (sqlite3_bind_int(pStmt, 2, (int)(uid)) != SQLITE_OK) 
	{
    		WriteLog(NameFunction, "Error: Couldn't bind parametr #1 !\n");
	        return -1;
	}

	if (sqlite3_bind_text(pStmt, 3, text, -1, SQLITE_TRANSIENT) != SQLITE_OK)
	{
    		WriteLog(NameFunction, "Error: Couldn't bind parametr #2 !\n");
	        return -1;
	}

	if (sqlite3_bind_text(pStmt, 4, date, -1, SQLITE_TRANSIENT) != SQLITE_OK)
	{
	        WriteLog(NameFunction, "Error: Couldn't bind parametr #3 !\n");
    		return -1;
	}

        rc = sqlite3_step(pStmt);
	if (rc != SQLITE_DONE) 
	{
    		WriteLog(NameFunction,  "Error: %s!\n", sqlite3_errmsg(db));
	        return rc;
	}

	rc = sqlite3_last_insert_rowid(db);
        if (rc < 0)
	{
    		WriteLog(NameFunction,  "Error: It's not insert row!\n");
	}
        sqlite3_reset(pStmt);
	sqlite3_finalize(pStmt);
	
	return 0;
}
#undef NameFunction


#define NameFunction "impr_sql_message_get_last\t"
int
impr_sql_message_get_last	(int uid, char *text_out)
{
	char request[512], *value = NULL;
	sqlite3_stmt* pStmt;
	int rc, coln, size;

	sprintf (request, "SELECT text FROM message WHERE mid in (SELECT MAX (mid) FROM message WHERE uid = %d) ;", uid);
	rc = prepare_requst(&db, request, &pStmt);
        if (rc != SQLITE_OK)
	{
	    	return -1;
	}

        while ((rc = sqlite3_step(pStmt)) == SQLITE_ROW) 
	{
	        coln = sqlite3_data_count(pStmt);
	        if (coln == 1) 
		{
	    		size = sqlite3_column_bytes(pStmt, 0);
	                value = malloc(size);
        		if (!value) 
			{
	                    WriteLog(NameFunction, "Error: Don't malloc memory!\n");
        	            return -1;
	                }	
			bzero (value, size);
        		memcpy(value, (void *)sqlite3_column_text(pStmt, 0), size);
			value[size] = '\0';
	        }
        }

	sprintf (text_out, "%s", value);
        sqlite3_reset(pStmt);
	sqlite3_finalize(pStmt);

	free (value);
	return 0;
}
#undef NameFunction

int
impr_sql_message_delete		(int mid)
{
	return 0;
}


int
impr_sql_message_delete_all	(int uid)
{
	char request[512];
	int rc;

	sprintf (request, "DELETE FROM message WHERE uid = %d;", uid);
	rc = exec_request_db(&db, request);
        if (rc != SQLITE_OK)
	{
    		return rc;
	}

	return 0;
}


/* Testplans operations */
#define NameFunction "impr_sql_testplan_add"
int
impr_sql_testplan_add 		()
{
	char request[128] = "\0";
	int rc;
	sqlite3_stmt* pStmt;

	sprintf (request, "INSERT INTO testplan VALUES (?, ?, ?, NULL, ?, ? ,? ,? ,? ,? ,? ,? ,? ,? ,? ,?, ?);");
	
	rc = prepare_requst(&db, request, &pStmt);
	if (rc != SQLITE_OK)
	{
    		return -1;
	}

        rc = sqlite3_step(pStmt);
        if (rc != SQLITE_DONE) 
	{
	        WriteLog(NameFunction,  "Error: %s!\n", sqlite3_errmsg(db));
    		return rc;
	}

	rc = sqlite3_last_insert_rowid(db);
        if (rc < 0)
	{
    		WriteLog(NameFunction,  "Error: It's not insert row!\n");
	}

	sqlite3_reset(pStmt);
        sqlite3_finalize(pStmt);

	return rc;
}
#undef NameFunction


int 
fillfield 	(sqlite3_stmt* pStmt, 
		 int coli, 
		 char *string_out) 
{
	int size;
	char *value;

	bzero (string_out, strlen (string_out));
	size = sqlite3_column_bytes (pStmt, coli);
	value = malloc(size+1);
	if (!value) 
	{
    		WriteLog("fillfield", "Error: Don't malloc memory!\n");
	        return -1;
	}

    	bzero (value, size + 1);
        memcpy(value, (void *)sqlite3_column_text(pStmt, coli), size);
	value[size] = '\0';
	sprintf (string_out, "%s", value);

	free (value);
	return 0;
}

int
print_testplan (testplan_st testplan)
{
	printf ("testplan:\n1. TPPK = %d\n2. TPID = %d\n3. UID = %d\n4. NAME = %s\n5. DESCRIPTION = %s\n6. GROUP = %s\n7. STATUS = %s\n8. PRIOR = %d\n9. TIMEDONWLOAD = %s\n10. TIMSTART = %s\n11. TIMEFINISH=%s\n12. TIMESEND=%s\n13. QUESTIONNUM = %d\n14. CURQUESTION = %d\n", testplan.tppk, testplan.tpid, testplan.uid, testplan.name, testplan.description, testplan.group, testplan.status, testplan.prioritet, testplan.downloadtime, testplan.starttime, testplan.finishtime, testplan.sendtime, testplan.questionnum, testplan.curquestion);

	return 0;
}

#define NameFunction "impr_sql_testplan_init_struct"
int
impr_sql_testplan_init_struct (int tpid, testplan_st *testplan_out)
{
	char *request = NULL;
	int rc, coln;
	sqlite3_stmt* pStmt;

	request = sqlite3_mprintf ("SELECT * FROM testplan WHERE tpid = %d;",tpid);
	rc = prepare_requst(&db, request, &pStmt);
        if (rc != SQLITE_OK)
	{
		return -1;
	}	

        while ((rc = sqlite3_step(pStmt)) == SQLITE_ROW) 
	{
	        coln = sqlite3_data_count(pStmt);

    		if (coln == 17) 
		{
			fillfield (pStmt, 0, testplan_out->wish_file);
			fillfield (pStmt, 1, testplan_out->wish_text);
			testplan_out->questionnumdone = sqlite3_column_int (pStmt, 2);
			testplan_out->curquestion = sqlite3_column_int (pStmt, 3);
			testplan_out->tppk = sqlite3_column_int (pStmt, 4);
			fillfield (pStmt, 5, testplan_out->group);
			testplan_out->tpid = tpid;
			fillfield (pStmt, 7, testplan_out->name);
			fillfield (pStmt, 8, testplan_out->description);
			fillfield (pStmt, 9, testplan_out->status);
			testplan_out->prioritet = sqlite3_column_int (pStmt, 10);
			fillfield (pStmt, 11, testplan_out->starttime);
			fillfield (pStmt, 12, testplan_out->finishtime);
			fillfield (pStmt, 13, testplan_out->sendtime);
			fillfield (pStmt, 14, testplan_out->downloadtime);
			testplan_out->questionnum = sqlite3_column_int (pStmt, 15);
			testplan_out->uid = sqlite3_column_int (pStmt, 16);
    		}
		else
		{
			WriteLog(NameFunction, "Wrong cols result!\n");
		}
        }

	sqlite3_free (request);
        sqlite3_reset(pStmt);
	sqlite3_finalize(pStmt);

	return 0;
}
#undef NameFunction


#define NameFunction "impr_sql_testplan_update"
int
impr_sql_testplan_update (testplan_st testplan)
{
	char *request = NULL;
	int rc;
	
	request = sqlite3_mprintf ("UPDATE testplan SET curquestion = %d, numquestiondone = %d WHERE tppk = %d;", testplan.curquestion, testplan.questionnumdone, testplan.tppk);	

	rc = exec_request_db(&db, request);
	if (rc != SQLITE_OK)
	{
		WriteLog(NameFunction,  "Error: %s!\n", sqlite3_errmsg(db));
		return rc;
	}

	sqlite3_free (request);

	return 0;
}
#undef NameFunction


#define NameFunction "impr_sql_testplan_update_status"
int
impr_sql_testplan_update_status (testplan_st testplan, char *status)
{
	char *zSQL;
	int rc;
	
	zSQL = sqlite3_mprintf ( "UPDATE testplan SET status = %Q WHERE tpid = %d AND uid = %d;", status, testplan.tpid, testplan.uid);	

	rc = exec_request_db(&db, zSQL);
        if (rc != SQLITE_OK)
	{
		WriteLog(NameFunction,  "Error: %s!\n", sqlite3_errmsg(db));
		return rc;
	}

	sqlite3_free(zSQL);
	return 0;
}
#undef NameFunction


#define NameFunction "impr_sql_testplan_update_wish"
int
impr_sql_testplan_update_wish (testplan_st testplan)
{
	char *zSQL = NULL;
	int rc = 0;
	
	zSQL = sqlite3_mprintf ( "UPDATE testplan SET wish_text = %Q, wish_file = %Q WHERE tpid = %d AND uid = %d;", testplan.wish_text, testplan.wish_file, testplan.tpid, testplan.uid);	
	
	rc = exec_request_db(&db, zSQL);
	if (rc != SQLITE_OK)
	{
		WriteLog(NameFunction,  "Error: %s!\n", sqlite3_errmsg(db));
		return rc;
	}

	sqlite3_free(zSQL);
	
	return 0;
}
#undef NameFunction


#define NameFunction "impr_sql_testplans_update_status_to_dowbload"
int
impr_sql_testplans_update_status_to_dowbload ()
{
	char *zSQL;
	int rc;

	zSQL = sqlite3_mprintf ( "UPDATE testplan SET status = \'DOWNLOADED\' WHERE status = \'NEW\';");

	rc = exec_request_db(&db, zSQL);
        if (rc != SQLITE_OK)
	{
		WriteLog(NameFunction,  "Error: %s!\n", sqlite3_errmsg(db));
		return rc;
	}

	sqlite3_free(zSQL);
	return 0;
}
#undef NameFunction


int
impr_sql_testplan_init_db 	(int tppk, 
				 int uid, 
				 testplan_st testplan_in)
{
	char request[8192] = "\0", *zSQL = NULL;
	int rc;

	zSQL = sqlite3_mprintf ("UPDATE testplan SET tpid = %d, uid = %d, name = %Q, description = %Q, numquestions = %d, group_ = %Q, status = %Q, prior = %d, timedownload = %Q, curquestion = %d, numquestiondone = %d WHERE tppk = %d;", testplan_in.tpid, testplan_in.uid, testplan_in.name, testplan_in.description, testplan_in.questionnum, testplan_in.group, testplan_in.status, testplan_in.prioritet, testplan_in.downloadtime, testplan_in.curquestion, testplan_in.questionnumdone,tppk);

	rc = exec_request_db(&db, zSQL);
        if (rc != SQLITE_OK)
	{
		return rc;
	}
	sqlite3_free(zSQL);

	return 0;
}


int
impr_sql_testplan_edit_download_time (int tpid, int uid)
{
	char *request = NULL, time_str[128];
	int rc;
	time_t result;

	result = time(NULL);
	strftime (time_str, 128, "%FT%T", localtime(&result));

	request = sqlite3_mprintf ("UPDATE testplan SET timedownload = \"%s\" WHERE tpid = %d AND uid = %d;", time_str, tpid, uid);	

	rc = exec_request_db(&db, request);
        if (rc != SQLITE_OK)
	{
		return rc;
	}

	sqlite3_free (request);

	return 0;
}


int
impr_sql_testplan_edit_start_time (int tpid, int uid)
{
	char *request = NULL, time_str[128];
	int rc;
	time_t result;

	result = time(NULL);
	strftime (time_str, 128, "%FT%T", localtime(&result));

	request = sqlite3_mprintf ("UPDATE testplan SET timestart = \"%s\" WHERE tpid = %d AND uid = %d;", time_str, tpid, uid);	

	rc = exec_request_db(&db, request);
        if (rc != SQLITE_OK)
	{
		return rc;
	}

	sqlite3_free (request);

	return 0;
}


int
impr_sql_testplan_edit_finish_time (int tpid, int uid)
{
	char *request = NULL, time_str[128];
	int rc;
	time_t result;

	result = time(NULL);
	strftime (time_str, 128, "%FT%T", localtime(&result));

	request = sqlite3_mprintf ("UPDATE testplan SET timefinish = \"%s\" WHERE tpid = %d AND uid = %d;", time_str, tpid, uid);	
	
	rc = exec_request_db(&db, request);
        if (rc != SQLITE_OK)
	{
		return rc;
	}

	sqlite3_free (request);

	return 0;

}


int
impr_sql_testplan_edit_send_time (int tpid, int uid)
{
	char *request = NULL, time_str[128];
	time_t result;
	int rc;

	result = time(NULL);
	strftime (time_str, 128, "%FT%T", localtime(&result));

	request = sqlite3_mprintf ("UPDATE testplan SET timesend = \"%s\" WHERE tpid = %d AND uid = %d;", time_str, tpid, uid);	
		
	rc = exec_request_db(&db, request);
        if (rc != SQLITE_OK)
	{
		return rc;
	}

	sqlite3_free (request);

	return 0;
}


int
impr_sql_testplan_edit_status (int tpid, int uid, char *status)
{
	char *request = NULL;
	int rc;

	sprintf (request, "UPDATE testplan SET status = \"%s\" WHERE tpid = %d AND uid = %d;", status, tpid, uid);	

	rc = exec_request_db(&db, request);
        if (rc != SQLITE_OK)
	{
	    	return rc;
	}

	sqlite3_free (request);

	return 0;
}


int 
impr_sql_testplan_delete (int tpid, int uid)
{
	char *request = NULL;
	int rc;

	request = sqlite3_mprintf ("DELETE FROM testplan WHERE uid = %d AND tpid = %d;", uid, tpid);
        rc = exec_request_db(&db, request);
	if (rc != SQLITE_OK)
	{
    		return rc;
	}

	sqlite3_free (request);

	return 0;
}


#define NameFunction "impr_sql_testplan_get_list"
int
impr_sql_testplan_get_list (int uid, 
			    int (*use_group)(char *gname, void* gdata), 
			    int (*use_testplan)(testplan_st testplan, void *gdata), 
			    void *data_)
{
	char *request, *value = NULL, *request2;
	int rc, rc_t, coln, coln_t, size = 0;
	sqlite3_stmt* pStmt, *pStmt_t;
	testplan_st testplan_buf;

	request = sqlite3_mprintf ("SELECT distinct (group_) FROM testplan WHERE uid = %d;", uid);
	rc = prepare_requst(&db, request, &pStmt);

    	if (rc != SQLITE_OK)
	{
    		return -1;
	}	

	while ((rc = sqlite3_step(pStmt)) == SQLITE_ROW) 
	{
    		coln = sqlite3_data_count(pStmt);

	        if (coln == 1) 
		{
			size = sqlite3_column_bytes (pStmt, 0);
			value = malloc(size+1);
			if (!value) 
			{
				WriteLog("fillfield", "Error: Don't malloc memory!\n");
				return -1;
			}
	
			bzero (value, size+1);
			memcpy(value, (void *)sqlite3_column_text(pStmt, 0), size);
			value[size] = '\0';

			use_group (value, data_);
			request2 = sqlite3_mprintf ("SELECT * FROM testplan WHERE uid = %d AND group_ = %Q;", uid, value);
			free (value);
			rc_t = prepare_requst(&db, request2, &pStmt_t);
			while ((rc_t = sqlite3_step(pStmt_t)) == SQLITE_ROW) 
			{
				coln_t = sqlite3_data_count(pStmt_t);

				if (coln_t == 17) 
				{
					fillfield (pStmt_t, 0, testplan_buf.wish_file);
					fillfield (pStmt_t, 1, testplan_buf.wish_text);					
					testplan_buf.questionnumdone = sqlite3_column_int (pStmt_t, 2);
					testplan_buf.curquestion = sqlite3_column_int (pStmt_t, 3);
					testplan_buf.tppk = sqlite3_column_int (pStmt_t, 4);
					fillfield (pStmt_t, 5, testplan_buf.group);
					testplan_buf.tpid = sqlite3_column_int (pStmt_t, 6);
					fillfield (pStmt_t, 7, testplan_buf.name);
					fillfield (pStmt_t, 8, testplan_buf.description);
					fillfield (pStmt_t, 9, testplan_buf.status);
					testplan_buf.prioritet = sqlite3_column_int (pStmt_t, 10);			
					fillfield (pStmt_t, 11, testplan_buf.starttime);
					fillfield (pStmt_t, 12, testplan_buf.sendtime);
					fillfield (pStmt_t, 13, testplan_buf.finishtime);
					fillfield (pStmt_t, 14, testplan_buf.downloadtime);
					testplan_buf.questionnum = sqlite3_column_int (pStmt_t, 15);
					testplan_buf.uid = sqlite3_column_int (pStmt_t, 16);
					testplan_buf.uid = uid;
					use_testplan (testplan_buf, data_);
				}
				else
				{
					WriteLog(NameFunction, "Wrong cols result!\n");
				}
			}

			sqlite3_reset(pStmt_t);
			sqlite3_finalize(pStmt_t);
			sqlite3_free(request2);
		}
	}
	sqlite3_free(request);
	sqlite3_reset(pStmt);
	sqlite3_finalize(pStmt);

	return 0;
}
#undef NameFunction 


#define NameFunction "impr_sql_testplan_get_list"
int
impr_sql_testplan_get_list_not_sended (int uid, int *imas)
{
	char *request, *value = NULL;
	int rc, rc_t, coln, coln_t, size = 0;
	sqlite3_stmt* pStmt;
	testplan_st testplan_buf;
	int i = 0;

	request = sqlite3_mprintf ("SELECT tpid FROM testplan WHERE status = \"FINISHED\" AND uid = %d;", uid);
	rc = prepare_requst(&db, request, &pStmt);

        if (rc != SQLITE_OK)
	{
		return -1;
	}

	while ((rc = sqlite3_step(pStmt)) == SQLITE_ROW) 
	{
    		coln = sqlite3_data_count(pStmt);

	        if (coln == 1) 
		{
			imas[i++] = sqlite3_column_int (pStmt, 0);
		}
	}

	sqlite3_free(request);
	sqlite3_reset(pStmt);
	sqlite3_finalize(pStmt);

	return i;
}
#undef NameFunction 


/* Question operations */
#define NameFunction "impr_sql_question_add"
int
impr_sql_question_add		(int tpid, 
				 int uid, 
				 question_st question)
{
	char *request = NULL;
	sqlite3_stmt* pStmt;
	int rc, error;

	request = sqlite3_mprintf ("INSERT INTO question (hasown, comment, curnumattachs, uid, qid, item, name, description, screen, numscreens, attach, numattachs, sizeattachs, answered, type, qnum, tpid) VALUES (?, ?, ?,  ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?);");
	rc = prepare_requst(&db, request, &pStmt);
    
	if (rc != SQLITE_OK)
	{
    		return -1;
	}
	
	if ((error = sqlite3_bind_int(pStmt, COL_QUESTION_HASOWN, (int)(question.isown))) != SQLITE_OK) 
	{
	        WriteLog(NameFunction, "14 Error: Couldn't bind parametr #%d [%d] %d!\n", COL_QUESTION_HASOWN, question.isown, error);
    		return -1;
	}

	if (sqlite3_bind_text(pStmt, COL_QUESTION_COMMENT, question.comment, -1, SQLITE_TRANSIENT) != SQLITE_OK)
	{
    		WriteLog(NameFunction, "2 Error: Couldn't bind parametr #%d !\n", COL_QUESTION_COMMENT);
    		return -1;
	}


	if ((error = sqlite3_bind_int(pStmt, COL_QUESTION_CURNUM_ATTACH, (int)(question.curnumattachs))) != SQLITE_OK) 
	{
	        WriteLog(NameFunction, "15 Error: Couldn't bind parametr #%d !\n", COL_QUESTION_CURNUM_ATTACH);
	        return -1;
        }

	if (sqlite3_bind_int(pStmt, COL_QUESTION_UID, (int)(question.uid)) != SQLITE_OK) 
	{
    		WriteLog(NameFunction, "0. Error: Couldn't bind parametr #%d !\n", COL_QUESTION_UID);
	        return -1;
	}


	if (sqlite3_bind_int(pStmt, COL_QUESTION_ID, (int)(question.qid)) != SQLITE_OK) 
	{
    		WriteLog(NameFunction, "1 Error: Couldn't bind parametr #%d !\n", COL_QUESTION_ID);
	        return -1;
	}
	
	if (sqlite3_bind_text(pStmt, COL_QUESTION_ITEM, question.item, -1, SQLITE_TRANSIENT) != SQLITE_OK)
	{
    		WriteLog(NameFunction, "2 Error: Couldn't bind parametr #%d !\n", COL_QUESTION_ITEM);
	        return -1;
	}


	if (sqlite3_bind_text(pStmt, COL_QUESTION_NAME, question.name, -1, SQLITE_TRANSIENT) != SQLITE_OK)
	{
	        WriteLog(NameFunction, "3 Error: Couldn't bind parametr #%d !\n", COL_QUESTION_NAME);
    		return -1;
	}

	if (sqlite3_bind_text(pStmt, COL_QUESTION_DESCRIPTION, question.description, -1, SQLITE_TRANSIENT) != SQLITE_OK)
	{
    		WriteLog(NameFunction, "4 Error: Couldn't bind parametr #%d !\n", COL_QUESTION_DESCRIPTION);
	        return -1;
	}

	if (sqlite3_bind_text(pStmt, COL_QUESTION_SCREEN, question.screen, -1, SQLITE_TRANSIENT) != SQLITE_OK)
	{
    		WriteLog(NameFunction, "5 Error: Couldn't bind parametr #%d !\n", COL_QUESTION_SCREEN);
	        return -1;
	}

	if (sqlite3_bind_int(pStmt, COL_QUESTION_NUMSCREEN, (int)(question.numscreens)) != SQLITE_OK) 
	{
	        WriteLog(NameFunction, "6 Error: Couldn't bind parametr #%d !\n", COL_QUESTION_NUMSCREEN);
	        return -1;
	}

	if (sqlite3_bind_text(pStmt, COL_QUESTION_ATTACH, question.attach, -1, SQLITE_TRANSIENT) != SQLITE_OK)
	{
	        WriteLog(NameFunction, "7 Error: Couldn't bind parametr #%d !\n", COL_QUESTION_ATTACH);
    		return -1;
        }

	if (sqlite3_bind_int(pStmt, COL_QUESTION_NUMATTACH, (int)(question.numattachs)) != SQLITE_OK) 
	{
	        WriteLog(NameFunction, "8 Error: Couldn't bind parametr #%d !\n", COL_QUESTION_NUMATTACH);
	        return -1;
	}
	
	if (sqlite3_bind_int(pStmt, COL_QUESTION_SIZEATTACH, (int)(question.sizeattachs)) != SQLITE_OK) 
	{
	        WriteLog(NameFunction, "9 Error: Couldn't bind parametr #%d !\n", COL_QUESTION_SIZEATTACH);
	        return -1;
        }

	if (sqlite3_bind_int(pStmt, COL_QUESTION_ANSWERED, (int)(question.answered)) != SQLITE_OK) 
	{
	        WriteLog(NameFunction, "10 Error: Couldn't bind parametr #%d !\n", COL_QUESTION_ANSWERED);
	        return -1;
        }

	if (sqlite3_bind_text(pStmt, COL_QUESTION_TYPE, question.type, -1, SQLITE_TRANSIENT) != SQLITE_OK)
	{
	        WriteLog(NameFunction, "11 Error: Couldn't bind parametr #%d !\n", COL_QUESTION_TYPE);
	        return -1;
	}

	if (sqlite3_bind_int(pStmt, COL_QUESTION_NUM, (int)(question.qnum)) != SQLITE_OK) 
	{
	        WriteLog(NameFunction, "12 Error: Couldn't bind parametr #%d !\n", COL_QUESTION_NUM);
    		return -1;
	}

	if ((error = sqlite3_bind_int(pStmt, COL_QUESTION_TPID, (int)(question.tpid))) != SQLITE_OK) 
	{
	        WriteLog(NameFunction, "13 Error: Couldn't bind parametr #%d %d!\n", COL_QUESTION_TPID, error);
    		return -1;
        }

        rc = sqlite3_step(pStmt);
        if (rc != SQLITE_DONE) 
        {
    		WriteLog(NameFunction,  "Error: %s!\n", sqlite3_errmsg(db));
    		return rc;
	}

	rc = sqlite3_last_insert_rowid(db);
        if (rc < 0)
	{
    		WriteLog(NameFunction,  "Error: It's not insert row!\n");
	}

	sqlite3_free (request);
	sqlite3_reset(pStmt);
        sqlite3_finalize(pStmt);

	return 0;
}
#undef NameFunction

int
impr_sql_question_delete (int tpid, int uid)
{
	char *request = NULL;
	int rc;

	sprintf (request, "DELETE FROM question WHERE uid = %d AND tpid = %d;", uid, tpid);
        rc = exec_request_db(&db, request);
	if (rc != SQLITE_OK)
	{
    		return rc;
	}
	
	sqlite3_free (request);

	return 0;
}

int
impr_sql_question_edit (question_st question)
{
	char *zSQL = NULL;
	int rc;

	if (question.hascomment)
	{
		if (strlen (question.comment) > 0)
		{
	
			zSQL = sqlite3_mprintf ("UPDATE question SET answered = %d, curnumattachs = %d, comment = %Q WHERE tpid = %d AND qnum = %d;", question.answered, question.curnumattachs, question.comment, question.tpid, question.qnum);	
		}
		else
		{
			zSQL = sqlite3_mprintf ("UPDATE question SET answered = %d, curnumattachs = %d, comment = \"\" WHERE tpid = %d AND qnum = %d;", question.answered, question.curnumattachs, question.tpid, question.qnum);	
		}
	}
	else
	{
		zSQL = sqlite3_mprintf ("UPDATE question SET answered = %d, curnumattachs = %d WHERE tpid = %d AND qnum = %d;", question.answered, question.curnumattachs, question.tpid, question.qnum);	

	}

	rc = exec_request_db(&db, zSQL);
        if (rc != SQLITE_OK)
	{
		WriteLog("question_edit",  "Error: %s!\n", sqlite3_errmsg(db));
		return rc;
	}
	sqlite3_free(zSQL);

	return 0;

}


int
print_question (question_st question)
{
	printf ("question:\n1. QID = %d\n2. Q_UID = %d\n3. Q_TPID = %d\n4. Q_NUM = %d\n5. Q_NAME = %s\n6. Q_DESCR = %s\n7. Q_TYPE = %s\n8. Q_ATTACH = %s\n9. Q_SCREEN = %s\n10. Q_NUMATTACH = %d\n11. Q_SIZEATTACH = %d\n12. Q_NUMSCREEN = %d\n13. Q_COMMENT = %s\n14. Q_ANSWERED = %d\n15. Q_CURNUMATTACH = %d\n16. Q_ITEM = %s\n17. Q_ISOWN = %d\n18. Q_PK = %d\n",
question.qid, question.uid, question.tpid, question.qnum, question.name, question.description, question.type, question.attach, question.screen, question.numattachs, question.sizeattachs, question.numscreens, question.comment, question.answered, question.curnumattachs, question.item, question.isown, question.qpk);
	return 0;
}


#define NameFunction "impr_sql_question_init_struct"
int
impr_sql_question_init_struct	(int tpid, 
				 int uid, 
				 int qnum, 
				 question_st *question_out)
{
	char *request = NULL;
	int rc, coln;
	sqlite3_stmt* pStmt;

	request = sqlite3_mprintf ("SELECT * FROM question WHERE tpid = %d AND uid = %d AND qnum = %d;",tpid, uid, qnum);

	rc = prepare_requst(&db, request, &pStmt);
        if (rc != SQLITE_OK)
	{
		return -1;
	}

        while ((rc = sqlite3_step(pStmt)) == SQLITE_ROW) 
	{
	        coln = sqlite3_data_count(pStmt);
	        if (coln == 18) 
		{
			question_out->isown = sqlite3_column_int (pStmt, 0);
			question_out->qpk = sqlite3_column_int (pStmt, 3);
			question_out->uid = sqlite3_column_int (pStmt, 4);
			question_out->tpid = sqlite3_column_int (pStmt, 17);
			question_out->qid = sqlite3_column_int (pStmt, 5);

			question_out->curnumattachs = sqlite3_column_int (pStmt, 2);
			question_out->qnum = sqlite3_column_int (pStmt, 16);
			question_out->numattachs = sqlite3_column_int (pStmt, 12);

			question_out->numscreens = sqlite3_column_int (pStmt, 10);
			question_out->sizeattachs = sqlite3_column_int (pStmt, 13);
			question_out->answered = sqlite3_column_int (pStmt, 14);

			fillfield (pStmt, 9, question_out->screen);
			fillfield (pStmt, 11, question_out->attach);
			fillfield (pStmt, 6, question_out->item);

			fillfield (pStmt, 15, question_out->type);
			fillfield (pStmt, 7, question_out->name);
			fillfield (pStmt, 8, question_out->description);

			fillfield (pStmt, 1, question_out->comment);
	        }
		else
		{
			WriteLog(NameFunction, "Wrong cols result!\n");
		}
	}

	sqlite3_free (request);
        sqlite3_reset(pStmt);
	sqlite3_finalize(pStmt);

	return 0;
}
#undef NameFunction


#define NameFunction "impr_sql_attach_add"
int
impr_sql_attach_add		(int tpid, 
				 int uid, 
				 int qnum, 
				 char *fullpath)
{
	char *request = NULL;
	sqlite3_stmt* pStmt;
	int rc;

	request = sqlite3_mprintf ("INSERT INTO attach VALUES (?, ?, ?, ?, NULL)");
	
	rc = prepare_requst(&db, request, &pStmt);
        if (rc != SQLITE_OK)
	{
		return -1;
	}

	if (sqlite3_bind_int(pStmt, COL_ATTACH_UID, (int)(uid)) != SQLITE_OK) 
	{
    		WriteLog(NameFunction, "Error: Couldn't bind parametr #%d !\n", COL_ATTACH_UID);
	        return -1;
	}

	if (sqlite3_bind_int(pStmt, COL_ATTACH_TPID, (int)(tpid)) != SQLITE_OK) 
	{
    		WriteLog(NameFunction, "Error: Couldn't bind parametr #%d !\n", COL_ATTACH_TPID);
	        return -1;
	}

	if (sqlite3_bind_int(pStmt, COL_ATTACH_QNUM, (int)(qnum)) != SQLITE_OK) 
	{
	        WriteLog(NameFunction, "Error: Couldn't bind parametr #%d !\n", COL_ATTACH_QNUM);
	        return -1;
        }

	if (sqlite3_bind_text(pStmt, COL_ATTACH_PATH, fullpath, -1, SQLITE_TRANSIENT) != SQLITE_OK)
	{
	        WriteLog(NameFunction, "Error: Couldn't bind parametr #%d !\n", COL_ATTACH_PATH);
	        return -1;
        }

	rc = sqlite3_step(pStmt);
        if (rc != SQLITE_DONE) 
    	{
    		WriteLog(NameFunction,  "Error: %s!\n", sqlite3_errmsg(db));
	        return rc;
	}

        rc = sqlite3_last_insert_rowid(db);
	if (rc < 0)
	{
    		WriteLog(NameFunction,  "Error: It's not insert row!\n");
	}

	sqlite3_free (request);
        sqlite3_reset(pStmt);
	sqlite3_finalize(pStmt);

	return 0;
}
#undef NameFunction


int
impr_sql_attach_del			(int tpid, 
					 int qnum, 
					 char *fullpath)	
{
	char *zSQL;
	int rc;

	zSQL = sqlite3_mprintf ( "DELETE FROM attach WHERE tpid = %d AND qnum = %d AND fullpath = %Q;", tpid, qnum, fullpath);
	
	rc = exec_request_db(&db, zSQL);
        if (rc != SQLITE_OK)
	{
		return rc;
	}
	sqlite3_free(zSQL);

	return 0;
}


int
impr_sql_attach_del_with_testplan	(int tpid, int uid)	
{
	char *request = NULL;
	int rc;

	request = sqlite3_mprintf ("DELETE FROM attach WHERE tpid = %d AND uid = %d;", tpid, uid);
        rc = exec_request_db(&db, request);
	if (rc != SQLITE_OK)
	{
    		return rc;
	}

	sqlite3_free (request);

	return 0;
}


#define NameFunction "impr_sql_answer_add"
int
impr_sql_answer_add	(int tpid, 
			 int uid, 
			 int qnum, 
			 char *variant, 
			 int ismyself)
{
	char *request = NULL;
	sqlite3_stmt* pStmt;
	int rc, answer0 = 0;

	request = sqlite3_mprintf ("INSERT INTO answer VALUES (?, ?, ?, ?, ?, ?, NULL);");
	rc = prepare_requst(&db, request, &pStmt);
        if (rc != SQLITE_OK)
	{
		return -1;
	}

	if (sqlite3_bind_int(pStmt, COL_ANSWER_TPID, (int)(tpid)) != SQLITE_OK) 
	{
    		WriteLog(NameFunction, "0-> Error: Couldn't bind parametr #%d !\n", COL_ANSWER_TPID);
	        return -1;
	}


	if (sqlite3_bind_int(pStmt, COL_ANSWER_UID, (int)(uid)) != SQLITE_OK) 
	{
	        WriteLog(NameFunction, "1 Error: Couldn't bind parametr #%d !\n", COL_ANSWER_UID);
    		return -1;
	}

	if (sqlite3_bind_int(pStmt, COL_ANSWER_QNUM, (int)qnum) != SQLITE_OK)
	{
        	WriteLog(NameFunction, "2 Error: Couldn't bind parametr #%d !\n", COL_ANSWER_QNUM);
	        return -1;
	}


	if (sqlite3_bind_text(pStmt, COL_ANSWER_VARIANT, variant, -1, SQLITE_TRANSIENT) != SQLITE_OK)
	{
	        WriteLog(NameFunction, "3 Error: Couldn't bind parametr #%d !\n", COL_ANSWER_VARIANT);
    		return -1;
        }

	if (sqlite3_bind_int(pStmt, COL_ANSWER_ISMYSELF, (int)(ismyself)) != SQLITE_OK) 
	{
	        WriteLog(NameFunction, "6 Error: Couldn't bind parametr #%d !\n", COL_ANSWER_ISMYSELF);
	        return -1;
        }

	if (sqlite3_bind_int(pStmt, COL_ANSWER_ISCHOOSED, (int)(answer0)) != SQLITE_OK) 
	{
	        WriteLog(NameFunction, "6 Error: Couldn't bind parametr #%d !\n", COL_ANSWER_ISCHOOSED);
	        return -1;
	}


        rc = sqlite3_step(pStmt);
	if (rc != SQLITE_DONE) 
	{
    		WriteLog(NameFunction,  "Error: %s!\n", sqlite3_errmsg(db));
	        return rc;
        }

	rc = sqlite3_last_insert_rowid(db);
        if (rc < 0)
	{
	        WriteLog(NameFunction,  "Error: It's not insert row!\n");
	}

	sqlite3_free (request);
        sqlite3_reset(pStmt);
	sqlite3_finalize(pStmt);

	return 0;
}
#undef NameFunction


int 
impr_sql_answer_delete (int tpid, int uid)
{
	char *request = NULL;
	int rc;

	request = sqlite3_mprintf ("DELETE FROM answer WHERE tpid = %d AND uid = %d;", tpid, uid);
        rc = exec_request_db(&db, request);
	if (rc != SQLITE_OK)
	{
    		return rc;
	}

	sqlite3_free (request);

	return 0;
}


int
impr_sql_answer_zero (int tpid, int qnum)
{
	char *request = NULL;
	int rc;

	request = sqlite3_mprintf ("UPDATE answer SET ischoosed = 0 WHERE tpid = %d AND qnum = %d;", tpid, qnum);	

	rc = exec_request_db(&db, request);
        if (rc != SQLITE_OK)
	{
		return rc;
	}

	sqlite3_free (request);

	return 0;
}

int
impr_sql_answer_edit_choosen  	(char *answer,
				 int tpid,
				 int qnum)
{
	char *request = NULL;
	int rc;

	request = sqlite3_mprintf ("UPDATE answer SET ischoosed = 1 WHERE variant = %Q AND tpid = %d AND qnum = %d;", answer, tpid, qnum);	

	rc = exec_request_db(&db, request);
        if (rc != SQLITE_OK)
	{
		WriteLog("impr_sql_answer_edit_choosen",  "Error: %s!\n", sqlite3_errmsg(db));
		return rc;
	}

	sqlite3_free (request);
	return 0;
}


int
impr_sql_answer_edit_myself  	(char *answer, int tpid, int qnum)
{
	char *zSQL = NULL;
	int rc;

	zSQL = sqlite3_mprintf ( "UPDATE answer SET ischoosed = 1, variant = %Q WHERE ismyself = 1 AND tpid = %d AND qnum = %d;", answer, tpid, qnum);

	rc = exec_request_db(&db, zSQL);
	if (rc != SQLITE_OK)
	{
    		return rc;
	}
	sqlite3_free(zSQL);
	return 0;
}


#define NameFunction "impr_sql_answer_get_list"
int
impr_sql_answer_get_list 	(int tpid, 
				 int uid, 
				 int qnum, 
				 int (*use_answer) (int tpid_, int uid_, int qnum_, char *variant_, int ischoosen_, int ismyself_, int answer_num_, void *data_), 
				 void *data)
{
	char *request = NULL;
	int rc, coln, ischoosed = 0, ismyself = 0, i = 0;
	char variant[4096] = "\0";
	sqlite3_stmt* pStmt;

	request = sqlite3_mprintf ("SELECT * FROM answer WHERE tpid = %d AND uid = %d AND qnum = %d;", tpid, uid, qnum);	

	rc = prepare_requst(&db, request, &pStmt);

        if (rc != SQLITE_OK)
	{
		return -1;
	}

	while ((rc = sqlite3_step(pStmt)) == SQLITE_ROW) 
	{
    		coln = sqlite3_data_count(pStmt);
        	if (coln == 7) 
		{
			fillfield (pStmt, 3, variant);	
			ischoosed = sqlite3_column_int (pStmt, 4);
			ismyself = sqlite3_column_int (pStmt, 5 );

			use_answer (tpid, uid, qnum, variant, ischoosed, ismyself, i, data);
			i ++;
    		}
		else
		{
			WriteLog(NameFunction, "Wrong cols result!\n");
		}
	}

	sqlite3_free (request);
        sqlite3_reset(pStmt);
	sqlite3_finalize(pStmt);

	return i;
}
#undef NameFunction


#define NameFunction "impr_sql_attach_get_list"
int
impr_sql_attach_get_list (int tpid, int uid, int qnum, int (*use_attach) (int tpid_, int uid_, int qnum_, int attachid_, char *fullpath_, void *data_), void *data)
{
	char *request = NULL;
	int rc, coln, attachid = 0;
	char fullpath[512] = "\0";
	sqlite3_stmt* pStmt;

	request = sqlite3_mprintf ("SELECT * FROM attach WHERE tpid = %d AND uid = %d AND qnum = %d;", tpid, uid, qnum);	
	rc = prepare_requst(&db, request, &pStmt);
        if (rc != SQLITE_OK)
	{
		WriteLog(NameFunction,  "Error: %s!\n", sqlite3_errmsg(db));
		return -1;
	}	

        while ((rc = sqlite3_step(pStmt)) == SQLITE_ROW) 
	{
	        coln = sqlite3_data_count(pStmt);

	        if (coln == 5) 
		{
			fillfield (pStmt, 0, fullpath);
			use_attach (tpid, uid, qnum, attachid, fullpath, data);
    		}
		else
		{
			WriteLog(NameFunction, "Wrong cols result!\n");
		}
	}

	sqlite3_free (request);
        sqlite3_reset(pStmt);
	sqlite3_finalize(pStmt);

	return 0;
}
#undef NameFunction


#define NameFunction "impr_sql_attach_get_list"
int
impr_sql_attach_get_list_for_report (int tpid, 
				     int uid, int (*use_attach) (int tpid_, int uid_, int attachid_, char *fullpath_))
{
	char *request = NULL;
	int rc, coln, attachid = 0;
	char fullpath[512] = "\0";
	sqlite3_stmt* pStmt;

	request = sqlite3_mprintf("SELECT * FROM attach WHERE tpid = %d AND uid = %d", tpid, uid);
	rc = prepare_requst(&db, request, &pStmt);
        if (rc != SQLITE_OK)
	{
		WriteLog(NameFunction,  "Error: %s!\n", sqlite3_errmsg(db));
		return -1;
	}	

        while ((rc = sqlite3_step(pStmt)) == SQLITE_ROW) 
	{
	        coln = sqlite3_data_count(pStmt);

    		if (coln == 5) 
		{
			fillfield (pStmt, 0, fullpath);

			use_attach (tpid, uid, attachid, fullpath);
    		}
		else
		{
			WriteLog(NameFunction, "Wrong cols result!\n");
		}
	}

	sqlite3_free (request);
        sqlite3_reset(pStmt);
	sqlite3_finalize(pStmt);

	return 0;
}
#undef NameFunction


#define NameFunction "impr_sql_question_get_list"
int
impr_sql_question_get_list (int tpid, 
			    int uid, int (*use_question) (int tpid, char *item_, char* description_, int isanswered_, int qnum_, void *data_), 
			    void *data)
{
	char *request = NULL;
	int rc, coln, isanswered = 0, qnum;
	char item[32] = "\0", description[8192] = "\0";
	sqlite3_stmt* pStmt;

	request = sqlite3_mprintf ("SELECT item, description, answered, qnum FROM question WHERE tpid = %d AND uid = %d;", tpid, uid);	
	rc = prepare_requst(&db, request, &pStmt);

        if (rc != SQLITE_OK)
	{
		return -1;
	}

	while ((rc = sqlite3_step(pStmt)) == SQLITE_ROW) 
	{
    		coln = sqlite3_data_count(pStmt);

    		if (coln == 4) 
		{
			fillfield (pStmt, 0, item);
			fillfield (pStmt, 1, description);
			isanswered = sqlite3_column_int (pStmt, 2);
			qnum = sqlite3_column_int (pStmt, 3);

			use_question (tpid, item, description, isanswered, qnum, data);
    		}
		else
		{
			WriteLog(NameFunction, "Wrong cols result!\n");
		}
	}

	sqlite3_free (request);
	sqlite3_reset(pStmt);
	sqlite3_finalize(pStmt);

	return 0;
}
#undef NameFunction


#define NameFunction "impr_sql_delete_before_insert"
int
impr_sql_delete_before_insert (int tpid, int uid)
{
	char *request, *value = NULL;
	int rc, rc_t, coln, coln_t, size = 0;
	sqlite3_stmt* pStmt;
	testplan_st testplan_buf;
	int i = 0;

	request = sqlite3_mprintf ("SELECT tppk FROM testplan WHERE tpid = %d;", tpid);
	rc = prepare_requst(&db, request, &pStmt);

    	if (rc != SQLITE_OK)
	{
    		return -1;
	}
	
	while ((rc = sqlite3_step(pStmt)) == SQLITE_ROW) 
	{
	    	coln = sqlite3_data_count(pStmt);

   	     	if (coln == 1) 
		{
			impr_sql_testplan_delete (tpid, uid );
			impr_sql_question_delete (tpid, uid);
			impr_sql_answer_delete	 (tpid, uid);
			impr_sql_attach_del_with_testplan (tpid, uid);
		}
	}
	sqlite3_free(request);
	sqlite3_reset(pStmt);
	sqlite3_finalize(pStmt);

	return i;
}
#undef NameFunction 


#define NameFunction "impr_sql_testplan_find"
int
impr_sql_testplan_find (int tpid, int uid)
{
	char *request, *value = NULL;
	int rc, rc_t, coln, coln_t, size = 0;
	sqlite3_stmt* pStmt;
	testplan_st testplan_buf;
	int i = 0;

	request = sqlite3_mprintf ("SELECT tppk FROM testplan WHERE tpid = %d;", tpid);
	rc = prepare_requst(&db, request, &pStmt);

        if (rc != SQLITE_OK)
	{
		return -1;
	}
	while ((rc = sqlite3_step(pStmt)) == SQLITE_ROW) 
	{
    		coln = sqlite3_data_count(pStmt);

    		if (coln == 1) 
		{
			return 1;
		}
	}

	sqlite3_free(request);
	sqlite3_reset(pStmt);
	sqlite3_finalize(pStmt);

	return -1;
}
#undef NameFunction 
