#include <sqlite3.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

enum q_columns {
	COL_QUESTION_PK = 0,
	COL_QUESTION_HASOWN,
	COL_QUESTION_COMMENT,
	COL_QUESTION_CURNUM_ATTACH,

	COL_QUESTION_UID,
	COL_QUESTION_ID,
	COL_QUESTION_ITEM,

	COL_QUESTION_NAME,
	COL_QUESTION_DESCRIPTION,
	COL_QUESTION_SCREEN,

	COL_QUESTION_NUMSCREEN,
	COL_QUESTION_ATTACH,

	COL_QUESTION_NUMATTACH,
	COL_QUESTION_SIZEATTACH,

	COL_QUESTION_ANSWERED,
	COL_QUESTION_TYPE,
	COL_QUESTION_NUM,
        COL_QUESTION_TPID
};

enum attach_columns {
	COL_ATTACH_PATH=1,
	COL_ATTACH_QNUM,
	COL_ATTACH_UID,
	COL_ATTACH_TPID,
	COL_ATTACH_PK
};

enum answer_columns
{
	COL_ANSWER_UID = 1,
	COL_ANSWER_TPID,
	COL_ANSWER_QNUM,
	COL_ANSWER_VARIANT,
	COL_ANSWER_ISCHOOSED,
	COL_ANSWER_ISMYSELF,
	COL_ANSWER_AID
};


typedef struct testplan_struct {
		
	int tppk; /* Primart key*/
	int tpid; /* tp ident*/
	int uid;  /* user id*/

        char name			[4096]; /* testplan name */
	char description		[8192];  /*testplan description */
        char group			[4096]; /* testplan group */
	char status			[32];  /* testplan local status */
        int  prioritet;			/*not uses*/
	int  questionnum;
	int  questionnumdone;		/*num of questions that answered*/

	char downloadtime	[32];
	char starttime		[32];
	char finishtime		[32];
	char sendtime		[32];

	int curquestion;

	char wish_text		[8192];
	char wish_file		[512];
    
}testplan_st;


typedef struct question_struct {
	int qpk;
	int qid;
	int tpid;
	int uid;

	char name[256];
	char item[32];
	char description[4096];
	
	char screen[8];
	int  numscreens;

	char attach[8];
	int  numattachs;
	int  sizeattachs;

	int  curnumattachs;

	char type[32];
	int  answered;
	char comment[8192];

	int hascomment;
	int isown;
	int qnum;
	int numanswers;
}question_st;


typedef struct user_struct {
	int  uid;
	char name[256];
	char second[256];
	char email[256];
	char comment[256];
	int  reg;

	char hw[64];

	int pause;
}user_st;

sqlite3 *db;

int
impr_sql_init		(const char *dbname, sqlite3 **db);


/* User operations */

int
impr_sql_user_add	(int uid, 
			 char *hwid,
			 char *name,
			 char *lastname,
			 char *email);


int
impr_sql_user_init 	(int uid,
			 user_st *user,
			 char *name,
			 char *lastname,
			 char *email,
			 char *hwid);


/*has no body, mb in future*/
int
impr_sql_user_change_pass	(int uid, 
				 char *oldpass,
				 char *newpass);


int 
impr_sql_user_find			(int uid);


/* Messages operations -- this function are not used*/

int 
impr_sql_message_add		(int uid, char *text, char *time);


int
impr_sql_message_get_last	(int uid, char *text_out);


int
impr_sql_message_delete		(int mid);


int
impr_sql_message_delete_all	(int mid);


/* Testplans operations */

int
impr_sql_testplan_add 		();


int
impr_sql_testplan_init_db	 (int tppk, 
				  int uid,
				  testplan_st testplan_in);


int
impr_sql_testplan_init_struct 	(int tpid, testplan_st *testplan_out);


int
impr_sql_testplan_update 	(testplan_st testplan);

int
impr_sql_testplan_getlist 	(int uid, testplan_st *list);

int
impr_sql_testplan_edit_download_time (int tpid, int uid);


int
impr_sql_testplan_edit_start_time (int tpid, int uid);


int
impr_sql_testplan_edit_finish_time (int tpid, int uid);


int
impr_sql_testplan_edit_send_time (int tpid, int uid);


int
impr_sql_testplan_get_download_time 	(int tpid, 
				         int uid,
				    	 char *download_out);



int
impr_sql_testplan_get_start_time 	(int tpid, 
					 int uid,
					 char *start_out);


int
impr_sql_testplan_get_finish_time 	(int tpid,
					 int uid,
					 char *finish_out);


int
impr_sql_testplan_get_send_time 	(int tpid,
					 int uid,	
					 char *send_out);


int
impr_sql_testplan_edit_status 		(int tpid,
					 int uid,
					 char *status);


int 
impr_sql_testplan_delete 		(int tpid, int uid);

int
impr_sql_testplan_update_status 	(testplan_st testplan, char *status);

int
impr_sql_testplans_update_status_to_dowbload ();


/* Question operations */

int
impr_sql_question_add		(int tpid, 
				 int uid, 
				 question_st question);

int
impr_sql_question_init_struct 	(int tpid, 
				 int uid, 
				 int qnum,
				 question_st *question_out);

int
print_question 			(question_st question);

int
impr_sql_question_get_list 	(int tpid,
				 int uid,
				 int (*use_question) (int tpid, char *item_, char* description_, int isanswered_, int qnum_,void *data_),
				 void *data);

int
impr_sql_question_delete 	(int tpid, int uid);

int
impr_sql_question_edit 		(question_st question);

int
impr_sql_attach_add		(int tpid, int uid, int qnum, char *fullpath);


int
impr_sql_attach_del		(int tpid, int qnum, char *fullpath);

int
impr_sql_attach_del_with_testplan	(int tpid, int uid);


int
impr_sql_attachs_get		();


int
impr_sql_answer_add		(int tpid, 
				 int uid,
				 int qnum,
				 char *variant,
				 int ismyself);


int
impr_sql_answer_edit_choosen  	(char *answer,
				 int tpid,
				 int qnum);

int
impr_sql_answer_edit_myself  	(char *answer,
				 int tpid,
				 int qnum);

int
impr_sql_answer_zero 		(int tpid, int qnum);

int
impr_sql_question_get		(int tpid, 
				 int uid, 
				 int qnum /*, OUT*/);


int
impr_sql_question_set_answered (int tpid, 
				int uid, 
				int qnum, 
				int answered);

int
impr_sql_question_get_answered (int tpid,
				int uid,
				int qnum);


int
print_testplan 			(testplan_st testplan);

int
impr_sql_testplan_get_list 	(int uid,
				 int (*use_group)(char *gname, void* gdata), 
				 int (*use_testplan)(testplan_st testplan, void *gdata), 
				 void *data_);

int
impr_sql_testplan_get_list_not_sended (int uid, int *imas);

int
impr_sql_attach_get_list 	(int tpid, 
				 int uid, 
				 int qnum,
				 int (*use_attach) (int tpid_, int uid_, int qnum_, int attachid_, char *fullpath_, void *data_), 
				 void *data);

int
impr_sql_attach_get_list_for_report (int tpid, 
				     int uid, 
				     int (*use_attach) (int tpid_, int uid_, int attachid_, char *fullpath_));


int
impr_sql_answer_get_list 	(int tpid, 
				 int uid, 
				 int qnum, 
				 int (*use_answer) (int tpid_, int uid_, int qnum_, char *variant_, int ischoosen_, int ismyself_, int answer_num_, void *data_), 
				 void *data);


int
impr_sql_testplan_update_wish 	(testplan_st testplan);

int
impr_sql_delete_before_insert 	(int tpid, int uid);

int
impr_sql_testplan_find 		(int tpid, int uid);