//#include <config.h>

#include "log.h"

#include <time.h>
#include <sys/time.h>
#include <stdarg.h>
#include <stdlib.h>
#include <strings.h>

#ifdef __cplusplus
extern "C" {
#endif

enum LOG_MODE_DEBUG {
  LOG_NO = 0, // log is absent
  LOG_STDOUT, // log in stdout / forbidden for daemon
  LOG_FILE,   // log in output file
};

//extern int IS_LOG_DEBUG;
//extern FILE *error_log;

inline int get_time ( char *str, int len ) {
     struct timeval tv;
     struct tm* ptm;

     bzero (str, len);
     if (len < 256) {
         //WriteLog ("get_time","Error: input buf small [%d] < 256b \n", len);
         return -EXIT_FAILURE;
     }

     if ( gettimeofday(&tv, NULL) ) {
         //WriteLog ("get_time","Error: gettimeofday #%d type %s\n", errno, strerror(errno));
         return -EXIT_FAILURE;
     }

     if ( !( ptm = localtime (&tv.tv_sec) ) ) {
         //WriteLog ("get_time","Error: get local time: #%d type %s\n", errno, strerror(errno));
         return -EXIT_FAILURE;
     }

     if( !strftime(str, len, "%Y/%m/%d %H:%M:%S", ptm) ) {
         //WriteLog ("get_time", "Error: get date: #%d type %s\n", errno, strerror(errno));
         return -EXIT_FAILURE;
     }

     return EXIT_SUCCESS;
}

extern void WriteLog(const char *nameFunction, const char *message, ...) {
    char time_str[256];

    //if ( get_time(time_str) = EXIT_SUCCESS);
    get_time (time_str, 256);

    if (IS_LOG_DEBUG) {
        va_list listArgs;
        va_start(listArgs, message);
        switch(IS_LOG_DEBUG) {
            case (LOG_STDOUT): {
                    fprintf(stdout, "%s:%s:", time_str, nameFunction);
                    vfprintf(stdout, message, listArgs);
                    fprintf(stdout, "\n");
                    fflush(stdout);
                }
                break;
            case (LOG_FILE):
                if (error_log) {
                    fprintf(error_log, "%s:%s:", time_str, nameFunction);
                    vfprintf(error_log, message, listArgs);
                    fprintf(error_log, "\n");
                    fflush(error_log);
                }
                break;
            default:
                break;
        }
        va_end(listArgs);
    }
}

#ifdef __cplusplus
}
#endif


