#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#define BUFSIZE 4096
#define SOCKET_TRANS_OLD_VERSION 0

#include "rtrans.h"

#ifndef __LOG_H
#include "log.h"
#endif


#ifdef __cplusplus
extern "C" {
#endif



int
send_sec (int sockfd, char *instr, unsigned int size_int)
{
    int len, i = 0;
    unsigned int char_int;

    len = send (sockfd, &size_int, sizeof(size_int), 0);

    while (i != size_int)
    {
        char_int = instr[i];
        send (sockfd, &char_int, sizeof (char_int), 0);
        i++;
    }

    return EXIT_SUCCESS;
}


int
recv_sec (int sockfd, char *outstr)
{
    int recv_size, i = 0, char_int = 0, rc;
    char *buffer = NULL;

    rc = recv (sockfd, &recv_size, sizeof (recv_size), 0);
    buffer = (char*)malloc ( (recv_size+1)*sizeof(char));
//    printf ("recv_size: %d\n", recv_size);
    if (buffer == NULL)
    {
        WriteLog ("recv_sec", "ERROR: Cant allocate memory malloc!");
        return -EXIT_FAILURE;
    }

    bzero (buffer, recv_size+1);
    while (i != recv_size)
    {
        recv (sockfd, &char_int, sizeof (char_int), 0);
        buffer[i] = char_int;

        i++;
    }

    buffer [recv_size] = '\0';
    for (i = 0; i < recv_size; i++)
    {
        outstr[i] = buffer[i];
    }

    free (buffer);
    return recv_size;
}

int
recv_file (int sockfd, char *filename)
{
    FILE *f;
    char buffer[BUFSIZE+1];
    int recv_size = 0, rc;

    f = fopen (filename, "w+");
    if (!f)
    {
        exit (-EXIT_FAILURE);
    }

    while (1)
    {
        bzero (buffer, BUFSIZE+1);
        recv_size = recv_sec (sockfd, buffer);

        if (strcmp (buffer, "END_FILE") == 0)
        {
            break;
        }
        
        if (strcmp (buffer, "FAILED_FILE") == 0)
        {
            fclose (f);
            return EXIT_FAILURE;
        }
        rc = fwrite (buffer, 1, recv_size, f);
        fflush (f);
    }
    fclose (f);

    return EXIT_SUCCESS;
}


void
send_file (int sockfd, char *filename)
{
    FILE *f;
    size_t len = 0;
    char line[4097] = "\0";
    struct stat buf;


    if (stat (filename, &buf) != -1)
    {
        f = fopen (filename, "r");
//	printf ("%s\n", filename);
	if (!f)
	{
    	    WriteLog ("process_testplans", "ERROR: cant open file %s\n", filename);
	    return;
	}

        while (1) 
	{
    	    bzero (line, 4096);
            len = fread (line, 1, 4096, f);
	    line[len] = '\0';
//		puts ("YES YES");
            send_sec (sockfd, line, len);
	    if (len == 0)
    	    {
                send_sec (sockfd, "END_FILE", 8);
	        break;
    	    }

            if (len < 4096)
	    {
    	        send_sec (sockfd, "END_FILE", 8);
        	break;
            }
	}

	fclose (f);
    }
}

#if (SOCKET_TRANS_OLD_VERSION == 1)
int
send_sec (int sockfd, char *instr, unsigned int size_int)
{
    int len, i = 0;
    unsigned int char_int;

    len = send (sockfd, &size_int, sizeof(size_int), 0);

    while (i != size_int)
    {
        char_int = instr[i];
        send (sockfd, &char_int, sizeof (char_int), 0);
        i++;
    }

    return 0;
}

int
recv_sec (int sockfd, char *outstr)
{
    int recv_size, i = 0, char_int;
    char *buffer;

    recv (sockfd, &recv_size, sizeof (recv_size), 0);

    buffer = (char*)malloc ((recv_size+1)*sizeof(char));
    if (buffer == NULL)
    {
        WriteLog ("recv_sec", "ERROR: Cant allocate memory malloc!\n");
        return - EXIT_FAILURE;
    }
    bzero (buffer, recv_size+1);
    while (i != recv_size)
    {
        recv (sockfd, &char_int, sizeof (char_int), 0);
        buffer[i] = char_int;
        i++;
    }

    buffer [recv_size] = '\0';
    for (i = 0; i < recv_size; i++)
    {
        outstr[i] = buffer[i];
    }

    free (buffer);
    return recv_size;
}

int
recv_file (int sockfd, char *filename)
{
    FILE *f;
    char buffer[4097];
    int recv_size = 0;

    f = fopen (filename, "w+");
    if (!f)
    {
        exit (EXIT_FAILURE);
    }

    while (1)
    {
        bzero (buffer, BUFSZIE+1);
        recv_size = recv_sec (sockfd, buffer);
        if (strcmp (buffer, "END_FILE") == 0)
        {
            break;
        }

        if (strcmp (buffer, "FAILED_FILE") == 0)
        {
            fclose (f);
            return EXIT_FAILURE;
        }
        fwrite (buffer, 1, recv_size, f);
        fflush (f);
    }
    fclose (f);
    fsync (sockfd);
    return 0;
}

void
send_file (int sockfd, char *filename)
{
    FILE *f;
    size_t len = 0;
    char line[4097] = "\0";

    f = fopen (filename, "r");

    if (!f)
    {
        WriteLog ("precss_testplans", "ERROR: cant open file\n");
        send_sec (sockfd, "FAILED_FILE", 11);
    }

    while (1) 
    {
        bzero (line, 4096);
        len = fread (line, 1, 4096, f);
        line[len] = '\0';

        send_sec (sockfd, line, len);
        if (len == 0)
        {
            send_sec (sockfd, "END_FILE", 8);
            break;
        }

        if (len < 4096)
        {
            if (len == 0)
            {
                send_sec (sockfd, "END_FILE", 8);
                break;
            }
            else
            {
                send_sec (sockfd, "END_FILE", 8);
                break;
            }
        }
    }

    fsync (sockfd);
    fclose (f);
}
#endif // SOCKET_TRANS_OLD_VERSION == 0

#ifdef __cplusplus
}
#endif
