#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "support.h"
#include "state.h"

GtkListStore *
init_list ()
{
	GtkListStore *liststore;

	liststore = gtk_list_store_new (4,
					GDK_TYPE_PIXBUF,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_INT);

	return liststore;
}	

G_MODULE_EXPORT void
on_map_destroy	                	(GtkObject       *object,
                                        gpointer         user_data)
{
	map_status = 0;
}


void
add_columns_list	(GtkTreeView	*treeview)
{
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkTreeModel *model; 
	GtkCellRendererToggle *toggle;
	gboolean radio;

	gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), GTK_TREE_MODEL (GLS_map));

       	renderer = gtk_cell_renderer_toggle_new ();
       	model =        gtk_tree_view_get_model (treeview);
	renderer = gtk_cell_renderer_pixbuf_new ();
                         
	column = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_min_width (column, 30);
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "pixbuf", 0);
        gtk_tree_view_append_column (treeview, column);

	renderer = gtk_cell_renderer_text_new ();
       	column = gtk_tree_view_column_new_with_attributes (_("Item"), renderer, "text", 1, NULL);
       	gtk_tree_view_column_set_resizable (column, TRUE);
       	gtk_tree_view_append_column (treeview, column);

       	renderer = gtk_cell_renderer_text_new ();
       	column = gtk_tree_view_column_new_with_attributes (_("Description"), renderer, "text", 2, NULL);
       	gtk_tree_view_append_column (treeview, column);
       	gtk_tree_view_column_set_resizable (column, TRUE);
}

gint 
add_question_list 	(gint tpid, 
			gchar *item_, 
			gchar* description_, 
			gint isanswered_, 
			gint qnum_,
			gpointer data_)
{
	GtkTreeIter iter;
	GdkPixbuf *status_icon;
	gchar iconname[128];


	gtk_list_store_append (GLS_map, &iter);	

	if (isanswered_)
	{
		g_sprintf (iconname, "/usr/share/improver_client/pixmaps/green.png");
				      
	}
	else
	{
		g_sprintf (iconname, "/usr/share/improver_client/pixmaps/red.png");
			
	}

	status_icon = gdk_pixbuf_new_from_file (iconname, NULL);
	gtk_list_store_set (GLS_map, &iter, 0, status_icon,
						1, item_,	
						2, description_,
						3, qnum_,
						-1);

	return 1;
}


gboolean 
view_button_press_cb(GtkTreeView *view, GdkEventButton *ev, gpointer user_data)
{
	GtkTreeIter iter;
	GtkTreeSelection *selection = gtk_tree_view_get_selection (view);
	gint qnum;

	if (map_status && current_state == STATE_TEST)
	{
		 if (gtk_tree_selection_get_selected (selection, NULL, &iter))
		 {
			gtk_tree_model_get(GTK_TREE_MODEL (GLS_map), &iter, 3, &qnum,  -1);
			if (global_testplan.curquestion != qnum)
			{
				request_update_question ();
				global_testplan.curquestion = qnum;
				set_state_test (); 
			}
		 }
	}
	
	return FALSE;
}

G_MODULE_EXPORT void
view_button_pres_row_activated (GtkTreeView        *treeview,
                                        GtkTreePath        *path,
                                        GtkTreeViewColumn  *col,
                                        gpointer            userdata)
{
     GtkTreeIter iter;
     GtkTreeSelection *selection = gtk_tree_view_get_selection (treeview);
     gint qnum;
     
    if (map_status && current_state == STATE_TEST)
    {
	if (gtk_tree_selection_get_selected (selection, NULL, &iter))
        {
	    gtk_tree_model_get(GTK_TREE_MODEL (GLS_map), &iter, 3, &qnum,  -1);
	    if (global_testplan.curquestion != qnum)
	    {
		request_update_question ();
		global_testplan.curquestion = qnum;
		set_state_test (); 
	    }
        }
    }


/*
     if (gtk_tree_selection_get_selected (selection, NULL, &iter))
     {
         gtk_tree_model_get(GTK_TREE_MODEL (GTS_testplan), &iter, 3, &status, 4, &tpid, -1);
         impr_sql_testplan_init_struct (tpid, &global_testplan);
         set_state_testdescr ();
         g_free (status);
     }*/
}


GtkWidget *
CreateTestMap (gint from, gint tpid)
{	
	GtkWidget *widget, *window, *vbox, *vp;

	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	GLADE_HOOKUP_OBJECT(GTK_WIDGET (get_widget ("window1")), window, "window_testmap");
	gtk_window_set_default_size (GTK_WINDOW (window), 400, 300);
	g_signal_connect (window, "destroy",G_CALLBACK (on_map_destroy), NULL);
	gtk_window_set_title (GTK_WINDOW (window), _("Question list of test plan"));
	gtk_window_set_icon_from_file  (GTK_WINDOW (window), "/usr/share/improver_client/pixmaps/improver_client.png", NULL);


	map_status = 1;

	widget = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (widget),  GTK_POLICY_AUTOMATIC,  GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER (window), widget);

	vp = gtk_viewport_new (NULL, NULL);
	gtk_container_add (GTK_CONTAINER (widget), vp);

	widget = gtk_tree_view_new ();
//	g_signal_connect(widget, "button-press-event", G_CALLBACK(view_button_press_cb), NULL);
	g_signal_connect(widget, "row_activated", G_CALLBACK(view_button_pres_row_activated), NULL);

	GLADE_HOOKUP_OBJECT(GTK_WIDGET (get_widget ("window1")),widget, "treemap");
	gtk_container_add (GTK_CONTAINER (vp), widget);

	add_columns_list (GTK_TREE_VIEW (widget));
	gtk_list_store_clear (GTK_LIST_STORE (GLS_map));

	impr_sql_question_get_list (tpid, getuid (), add_question_list, NULL);
		
	gtk_widget_show_all (window);

	return window;
}


gboolean  edit_map_tree (GtkTreeModel *model,
			 GtkTreePath *path,
		         GtkTreeIter *iter,
		         gpointer data)
{
	gint qnum;
	GdkPixbuf *status_icon;
	gchar iconname[128];

	gtk_tree_model_get(model, iter, 3, &qnum,  -1);
	if (global_question.qnum == qnum)
	{
		if (global_question.answered)
		{
			g_sprintf (iconname, "/usr/share/improver_client/pixmaps/green.png");
		
		}
		else
		{
			g_sprintf (iconname, "/usr/share/improver_client/pixmaps/red.png");
		}

		status_icon = gdk_pixbuf_new_from_file (iconname, NULL);
		gtk_list_store_set (GTK_LIST_STORE (model), iter, 0, status_icon, -1);	
	}
	
	
	return FALSE;
}


