#include <gtk/gtk.h>

#include "support.h"
#include "state.h"



void set_color (GtkTreeViewColumn *col,
                GtkCellRenderer *renderer,
                GtkTreeModel    *model,
                GtkTreeIter     *iter,
                gpointer	user_data)
{
        guint color;
        gchar text[4096] = "\0";
	GdkColor color_rgb;

	gchar *status;
	gchar *name;
	gint isgroup;

        gtk_tree_model_get(model, iter, 1, &name, 2, &isgroup,3, &status, -1);

	if (isgroup == 1)
	{

		g_sprintf (text, "<b>%s</b>", name);
		g_object_set(renderer, "markup", text,
						         "cell-background", "#cdcdcd",
	                                                                         "cell-background-set", TRUE, 
		                                                                                 NULL);
		return;
	}
	else
	{
		if (!g_strcmp0 (status, "NEW"))
		{
			color_rgb.red = 64000;
			color_rgb.green = 10000;
			color_rgb.blue = 10000;

			g_object_set(renderer, "text", g_locale_to_utf8(name,-1,NULL,NULL,NULL),"foreground-set", FALSE,
				                 "cell-background", "#f4e3e1",
						  "cell-background-set", TRUE, 
		 	                           NULL);
			return;
		}

		if (!g_strcmp0 (status, "REPORTED"))
		{
			color_rgb.red = 64000;
			color_rgb.green = 10000;
			color_rgb.blue = 10000;

			g_object_set(renderer, "text", g_locale_to_utf8(name,-1,NULL,NULL,NULL),"foreground-set", FALSE,
//				                 "cell-background-gdk", &color_rgb,
				                 "cell-background", "#eff6f5",
						  "cell-background-set", TRUE, 
		 	                           NULL);
			return;
		}

		if (!g_strcmp0 (status, "DOWNLOADED"))
		{
			g_object_set(renderer, "text", g_locale_to_utf8(name,-1,NULL,NULL,NULL),"foreground-set", FALSE,
				                 "cell-background", "white",
		                                                                                 "cell-background-set", TRUE, 
		                                                                                 NULL);

		}

		if (!g_strcmp0 (status, "STARTED"))
		{
			g_object_set(renderer, "text", g_locale_to_utf8(name,-1,NULL,NULL,NULL),"foreground-set", FALSE,
				                 "cell-background", "white",
		                                                                                 "cell-background-set", TRUE, 
		                                                                                 NULL);

		}

		if (!g_strcmp0 (status, "FINISHED"))
		{
			g_object_set(renderer, "text", g_locale_to_utf8(name,-1,NULL,NULL,NULL),"foreground-set", FALSE,
				                 "cell-background", "white",
		                                                                                 "cell-background-set", TRUE, 
		                                                                                 NULL);

		}
	}

	g_free (name);
	g_free (status);
}


void
get_inoc_name (gchar *status, gchar *status_icon_name)
{
	if (!g_strcmp0 (status, "NEW"))
	{
		g_sprintf (status_icon_name,"/usr/share/improver_client/pixmaps/new.png");
	}

	if (!g_strcmp0 (status, "DOWNLOADED"))
	{
		g_sprintf (status_icon_name,"/usr/share/improver_client/pixmaps/downloaded.png");
	}

	if (!g_strcmp0 (status, "STARTED"))
	{
		g_sprintf (status_icon_name,"/usr/share/improver_client/pixmaps/started.png");
	}

	if (!g_strcmp0 (status, "FINISHED"))
	{
		g_sprintf (status_icon_name,"/usr/share/improver_client/pixmaps/finished.png");
	}

	if (!g_strcmp0 (status, "REPORTED"))
	{
		g_sprintf (status_icon_name,"/usr/share/improver_client/pixmaps/reported.png");
	}
}

void
add_columns	(GtkTreeView	*treeview, 
		 GtkTreeStore	*treestore)
{
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkTreeModel *model; 
	GtkCellRendererToggle *toggle;
	gboolean radio;

	gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), GTK_TREE_MODEL (treestore));
       	renderer = gtk_cell_renderer_toggle_new ();
	model =        gtk_tree_view_get_model (treeview);
	renderer = gtk_cell_renderer_pixbuf_new ();
                         
	column = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_min_width (column, 30);
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "pixbuf", 0);
        gtk_tree_view_append_column (treeview, column);

        renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new ();
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 1);
	gtk_tree_view_column_set_cell_data_func(column, renderer, set_color, NULL, NULL);	
        gtk_tree_view_append_column (treeview, column);
}


GtkTreeStore *
init_tree ()
{
	GtkTreeStore * TreeStore = gtk_tree_store_new (6,
			                        GDK_TYPE_PIXBUF,
			                        G_TYPE_STRING,
			                        G_TYPE_INT,
			                        G_TYPE_STRING,
			                        G_TYPE_INT,
			                        G_TYPE_INT);
	add_columns (GTK_TREE_VIEW (get_widget ("treeview_testplanlist")), TreeStore);
	return TreeStore;
}


gint
add_plan (testplan_st testplan, gpointer data)
{
	GtkTreeIter iter;
	GdkPixbuf *status_icon;
	gchar status_icon_name[512];

	get_inoc_name (testplan.status, status_icon_name);
	status_icon = gdk_pixbuf_new_from_file (status_icon_name, NULL);
	
	gtk_tree_store_append (GTS_testplan, &iter, data);
	gtk_tree_store_set (GTS_testplan, &iter, 
						0, status_icon, 
						1, testplan.name,
						2, 0,
						3, testplan.status,
						4, testplan.tpid, 
						5, testplan.curquestion, -1);
}


gint
add_group (gchar *gname, gpointer data)
{
	GtkTreeIter iter;


	gtk_tree_store_append (GTS_testplan, data, NULL);
	gtk_tree_store_set (GTS_testplan, data, 1,gname, 2, 1, -1);
}



void 
fill_tree ()
{
	GtkTreeIter iter;
	gtk_tree_store_clear (GTS_testplan);
	impr_sql_testplan_get_list (getuid (), add_group, add_plan, &iter);
	gtk_tree_view_expand_all (GTK_TREE_VIEW (get_widget ("treeview_testplanlist")));
	gtk_tree_view_set_enable_tree_lines (GTK_TREE_VIEW (get_widget ("treeview_testplanlist")), TRUE);
}

