#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/encoding.h>
#include <libxml/xmlwriter.h>
#include <glib.h>

#include "support.h"

#define MY_ENCODING "UTF-8"


xmlChar *
ConvertInput(const gchar *in, const gchar *encoding)
{
	xmlChar *out;
	gint ret;
	gint size;
	gint out_size;
	gint temp;
	xmlCharEncodingHandlerPtr handler;

	if (in == 0)
	{
        	return 0;
	}

	handler = xmlFindCharEncodingHandler(encoding);

	if (!handler) 
	{
	        printf("ConvertInput: no encoding handler found for '%s'\n",
	               encoding ? encoding : "");
	        return 0;
	}

    	size = (int) strlen(in) + 1;
    	out_size = size * 2 - 1;
    	out = (unsigned char *) xmlMalloc((size_t) out_size);

    	if (out != 0) 
	{
		temp = size - 1;
		ret = handler->input(out, &out_size, (const xmlChar *) in, &temp);
		if ((ret < 0) || (temp - size + 1)) 
		{
		    	if (ret < 0) 
			{
			        printf ("ConvertInput: conversion wasn't successful.\n");
		    	} 
			else 
			{
			        printf ("ConvertInput: conversion wasn't successful. converted: %i octets.\n", temp);
		    }

		    xmlFree(out);
		    out = 0;
	        } 
		else 
		{
            		out = (unsigned char *) xmlRealloc(out, out_size + 1);
            		out[out_size] = 0;  /*null terminating out */
        	}
	} 
	else 
	{
	        printf("ConvertInput: no mem\n");
    	}

	return out;
}


gint
get_attribute (xmlNodePtr cur, char *attr_name, char *out)
{
	xmlChar *key = NULL;
  
	key= xmlGetNoNsProp (cur, (const xmlChar *)attr_name);

	if (key != NULL)
	{
    		sprintf (out, "%s", key);
		xmlFree (key);		
		return 1;
	}

	xmlFree (key);
	return 0;
}


xmlChar *
get_tag	(xmlNodePtr	cur, gchar		*tagname)
{
	xmlChar *key;

	while (cur != NULL)
	{
		if (!xmlStrcmp(cur->name, (const xmlChar*)tagname))
		{
			key = xmlNodeGetContent (cur);
			return key;
		}
		cur = cur->next;
	}
	return NULL;
}

void
get_answer	(xmlNodePtr	cur, gint tpid, gint qnum)
{
	xmlChar *key;

	while (cur != NULL)
	{
		if (!xmlStrcmp(cur->name, (const xmlChar*)"answer"))
		{
			key = xmlNodeGetContent (cur);
			impr_sql_answer_add			(tpid, getuid (), qnum, key, 0);
		}
		cur = cur->next;
	}
}


int
get_id_testplan		(xmlNodePtr	cur)
{
	return atoi(get_tag (cur, "id"));
}


void
get_descr_testplan	(xmlNodePtr	cur, 
					 gchar *out)
{
	sprintf(out, "%s", get_tag (cur, "description"));
}

void
get_title_testplan	(xmlNodePtr	cur, 
					 gchar *out)
{
	sprintf(out, "%s", get_tag (cur, "title"));
}

void
get_name_testplan	(xmlNodePtr	cur, 
					 gchar *out)
{
	sprintf(out, "%s", get_tag (cur, "name"));
}

void
get_group_testplan	(xmlNodePtr	cur, 
					 gchar *out)
{
	sprintf(out, "%s", get_tag (cur, "group"));
}

void
set_download_time (gchar *out)
{
	gchar time_str[128];
	time_t result;

	result = time(NULL);
	strftime (time_str, 128, "%FT%T", localtime(&result));
	sprintf (out, "%s", time_str);
}


int
get_question_id (xmlNodePtr cur)
{
	return atoi(get_tag (cur, "id"));
}


void
get_question_name (xmlNodePtr cur, gchar *out)
{
	sprintf (out, "%s", get_tag (cur, "name"));
}


void
get_question_article (xmlNodePtr cur, gchar *out)
{
	sprintf (out, "%s", get_tag (cur, "article"));
}


void
get_question_descr (xmlNodePtr cur, gchar *out)
{
	sprintf (out, "%s", get_tag (cur, "question_descr"));
}


int
get_attachable_attr (xmlNodePtr cur)
{
	gchar str[20];
	gint rc;

	rc = get_attribute (cur, "attachable", str);
	if (rc)
	{
		if (!strcmp (str, "t"))
		{
			return 1;
		}

		return 0;
	}

	return 0;
}


int
get_attachbale_num_attr (xmlNodePtr cur)
{
	gchar str[20];
	gint rc;

	rc = get_attribute (cur, "attachable_num", str);
	if (rc)
	{
		return atoi (str);
	}

	return 1;
}


int
get_attachbale_size_attr (xmlNodePtr cur)
{
	gchar str[20];
	gint rc;

	rc = get_attribute (cur, "attachable_size", str);
	if (rc)
	{
		return atol (str);
	}

	return 10000000;
}


int
get_screenshot_attr (xmlNodePtr cur)
{
	gchar str[20];
	gint rc;

	rc = get_attribute (cur, "screenshot", str);
	if (rc)
	{
		if (!strcmp (str, "t"))
		{
			return 1;
		}
		return 0;
	}

	return 0;
}


int
get_screenshots_num_attr (xmlNodePtr cur)
{
	gchar str[20];
	gint rc;

	rc =  get_attribute (cur, "screenshot_num", str);
	if (rc)
	{
		return atoi (str);
	}

	return 0;
}


int
get_screenshots_size_attr (xmlNodePtr cur)
{
	gchar str[20];
	gint rc;

	rc =  get_attribute (cur, "screenshot_size", str);
	if (rc)
	{
		return atol (str);
	}

	return 1000000;
}


void
get_qtype_attr (xmlNodePtr cur, gchar *out)
{
	gchar str[20];
	gint rc;

	rc = get_attribute (cur, "multi", str);

	if (strcmp (str, "t") == 0)
	{
		sprintf (out, "multi");
	}
	else
	{
		sprintf (out, "single");
	}
}


int
get_isown_attr (xmlNodePtr cur)
{
	gchar str[20];
	gint rc;

	rc = get_attribute (cur, "hasown", str);
	if (rc)
	{
		if (!strcmp (str, "t"))
		{
			return 1;
		}
	}

	return 0;
}


void
get_answers	(xmlNodePtr	cur, gint tpid, gint qnum)
{
	xmlChar *key;
	xmlNodePtr	cur_child;
	gchar answer[4096] = "\0"; 

	while (cur != NULL)
	{
		if (!xmlStrcmp(cur->name, (const xmlChar*)"answers"))
		{
			cur_child = cur->xmlChildrenNode;
			get_answer	(cur_child, tpid, qnum);
		}
		cur = cur->next;
	}
}


int
get_questions (xmlDocPtr doc, xmlNodePtr cur, gint tpid) 
{
	xmlChar *key;
        cur = cur->xmlChildrenNode;
	xmlNodePtr cur_child;
        gint  questions_count = 0, rc;
	
	question_st for_add_question;

	for_add_question.tpid = tpid;
	for_add_question.uid  = getuid ();
	for_add_question.answered  = 0;
	for_add_question.curnumattachs = 0;
	bzero (for_add_question.comment, 8192);
//	sprintf (for_add_question.comment, " ");

	while (cur != NULL)
        {

   		if ((!xmlStrcmp(cur->name, (const xmlChar *)"question"))) 
		{
			cur_child = cur->xmlChildrenNode;

			for_add_question.qid = get_question_id 	(cur_child);
			get_question_article 	(cur_child, for_add_question.item);
			get_question_name 	(cur_child, for_add_question.name);
			get_question_descr 	(cur_child, for_add_question.description);
			rc = get_attachable_attr (cur);

			if (rc)
			{
				sprintf (for_add_question.attach, "true");
				for_add_question.numattachs = get_attachbale_num_attr (cur);
				for_add_question.sizeattachs = get_attachbale_size_attr (cur);
			}
			else
			{
				sprintf (for_add_question.attach, "false");	
			}

			rc = get_screenshot_attr (cur);
			if (rc)
			{
				sprintf (for_add_question.screen, "true");
				for_add_question.numscreens = get_screenshots_num_attr (cur);
			}
			else
			{
				sprintf (for_add_question.screen, "false");
			}

			get_qtype_attr (cur, for_add_question.type);
			for_add_question.isown = get_isown_attr (cur);

			questions_count ++;
			for_add_question.qnum  = questions_count;
			get_answers (cur->xmlChildrenNode, tpid, questions_count);
			impr_sql_question_add (tpid, getuid (), for_add_question);
			if (for_add_question.isown)
			{
				impr_sql_answer_add	(tpid, getuid (), for_add_question.qnum, "", 1);
			}
		}

   		cur = cur->next;
	}

	return questions_count;
}


int
read_xml (gchar *docname)
{
	xmlDocPtr doc;
	xmlNodePtr cur;
	testplan_st for_add_testplan;
	gint tppk, inbd;

	doc = xmlParseFile(docname);
	if (doc == NULL ) 
	{
    		return -1;
	}

        cur = xmlDocGetRootElement(doc);
	if (cur == NULL) 
	{
		xmlFreeDoc(doc);
	        return -1;
	}

        cur = cur->xmlChildrenNode;
	while (cur != NULL) 
	{
		if ( xmlStrcmp (cur->name, (const xmlChar *)"testplan_info" ) == 0)
		{
			for_add_testplan.tpid = get_id_testplan 	(cur->xmlChildrenNode);
			for_add_testplan.uid = getuid ();

			get_descr_testplan	(cur->xmlChildrenNode, for_add_testplan.description);
			get_group_testplan	(cur->xmlChildrenNode, for_add_testplan.group);
			get_title_testplan	(cur->xmlChildrenNode, for_add_testplan.name);

			inbd = impr_sql_testplan_find (for_add_testplan.tpid, getuid ());
		        if ( inbd == -1)
			{
		            tppk = impr_sql_testplan_add ();
		        }
		        else
		        {
				break;
//		    	    puts ("in db!");
		        }
//			impr_sql_delete_before_insert (for_add_testplan.tpid, for_add_testplan.uid);
		}

		if (inbd == -1)
		{
			if ( xmlStrcmp (cur->name, (const xmlChar *)"questions" ) == 0)
			{
				for_add_testplan.questionnum = get_questions (doc, cur, for_add_testplan.tpid);
			}
		}

		sprintf (for_add_testplan.status, "NEW");
		for_add_testplan.prioritet = 0;

		set_download_time (for_add_testplan.downloadtime);
		bzero (for_add_testplan.starttime, 32);
		bzero (for_add_testplan.finishtime, 32);
		bzero (for_add_testplan.sendtime, 32);
		for_add_testplan.curquestion = 1;
		for_add_testplan.questionnumdone = 0;
		bzero (for_add_testplan.wish_text, 8192);
		bzero (for_add_testplan.wish_file, 512);

    		cur = cur->next;
	}

	if (inbd == -1)
	{
		impr_sql_testplan_init_db (tppk, getuid (), for_add_testplan);
	}


        xmlFreeDoc(doc);

	return 0;
}


int 
generate_answer (gint tpid_, 
				 gint uid_, 
				 gint qnum_, 
		    		 gchar *variant_, 
				 gint ischoosen_, 
				 gint ismyself_, 
				 gint  answer_num_,
				 gpointer writer)
{
	xmlChar *tmp;
	gint rc;

	if (ischoosen_)
	{
		tmp = ConvertInput (variant_, MY_ENCODING);
		rc = xmlTextWriterWriteElement(writer, BAD_CAST "answer", tmp);
		if (rc < 0)
		{
			return;
		}
	}
}


int 
generate_attach (gint tpid_,
		 gint uid_,
		 gint qnum_,
		 gint attachid_,
		 gchar *fullpath_,
		 gpointer writer)
{
	xmlChar *tmp;
	gint rc;

	tmp = ConvertInput (fullpath_ + search_last_slash (fullpath_), MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "attachment", tmp);
	if (rc < 0)
	{
		return;
	}
}


gint 
generate_question  (gint 	tpid,
		    gchar 	*item_, 
		    gchar	*description_,
		    gint 	isanswered_, 
		    gint qnum_,
		    gpointer writer)
{
	question_st report_q;
	gchar text[128] = "\0";
	xmlChar *tmp;
	gint rc, uid;

	impr_sql_question_init_struct (tpid, getuid (), qnum_, &report_q);
	rc = xmlTextWriterStartElement(writer, BAD_CAST "question");
	uid = getuid ();

	sprintf (text, "%d", report_q.qid);
   	tmp = ConvertInput (text, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "id", tmp);
	if (rc < 0)
	{
		return;
	}

	tmp = ConvertInput (report_q.comment, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "comment", tmp);
	if (rc < 0)
	{
		return;
	}

	rc = xmlTextWriterStartElement(writer, BAD_CAST "answers");
        if (rc < 0) 
        {
		return;
       	}

	impr_sql_answer_get_list (tpid, uid, qnum_, generate_answer, writer);
	rc = xmlTextWriterEndElement(writer);

	rc = xmlTextWriterStartElement(writer, BAD_CAST "attachments");
        if (rc < 0) 
        {
		return;
       	}
	impr_sql_attach_get_list (tpid, uid, qnum_, generate_attach, writer);
	rc = xmlTextWriterEndElement(writer);
	rc = xmlTextWriterEndElement(writer);
}




void
xml_generate_report (gint tpid, gchar *filename)
{
	gint rc, i;
	xmlTextWriterPtr writer;
	xmlChar *tmp;
	gchar fio[128] = "\0", isonumber[256] = "\0", client_version[256] = "\0", text[128] = "\0";
	testplan_st report_tp;
	gchar buffer2[256] = "\0", locale_str[256] = "\0", locale_new[256] = "\0";
    


	rc = impr_sql_testplan_init_struct (tpid, &report_tp);
	bzero (filename, sizeof (filename));
	sprintf (filename, "%s/%s/reports/%s_%d_%s.xml", g_get_home_dir (), CACHE_DIR, user_info.hw, report_tp.tpid, report_tp.sendtime);

	if (rc)
	{
		g_printf ("error in impr_sql_testplan_init_struct\n");
		exit (0);
	}

	writer = xmlNewTextWriterFilename (filename, 0);
	rc = xmlTextWriterStartDocument (writer, NULL, MY_ENCODING, NULL);
	if (rc < 0)
	{
		return;
	}
	rc = xmlTextWriterStartElement (writer, BAD_CAST "testplan");
	if (rc < 0)
	{
		return;
	}

	sprintf (text, "%d",report_tp.tpid);
	tmp = ConvertInput (text, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "id_tp", tmp);
	if (rc < 0)
	{
		return;
	}

	tmp = ConvertInput (report_tp.downloadtime, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "time_testplan_getted", tmp);
	if (rc < 0)
	{
		return;
	}

	tmp = ConvertInput (report_tp.starttime, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "time_test_started", tmp);
	if (rc < 0)
	{
		return;
	}

	tmp = ConvertInput (report_tp.finishtime , MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "time_test_finished", tmp);
	if (rc < 0)
	{
		return;
	}

	tmp = ConvertInput (report_tp.sendtime , MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "time_report_send", tmp);
	if (rc < 0)
	{
		return;
	}

	tmp = ConvertInput (user_info.hw, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "system_id", tmp);
	if (rc < 0)
	{
		return;
	}

	if (!read_file ("/etc/isonumber", isonumber))
	{
		tmp = ConvertInput (isonumber, MY_ENCODING);
		rc = xmlTextWriterWriteElement(writer, BAD_CAST "isonumber", tmp);
		if (rc < 0)
		{
		return;
		}
	}

	sprintf (locale_str, "%s", getenv ("LANG"));
	i = 0;
        while (locale_str[i] != '.')
        {
                buffer2[i] = locale_str[i];
                i ++;
        }
        buffer2 [i+1] = '\0';

	sprintf (locale_new, "%s.utf8", buffer2);
	tmp = ConvertInput (locale_new, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "locale", tmp);
	if (rc < 0)
	{
		return;
	}

	tmp = ConvertInput (report_tp.wish_text, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "wish_text", tmp);
	if (rc < 0)
	{
		return;
	}

	tmp = ConvertInput (report_tp.wish_file + search_last_slash (report_tp.wish_file), MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "wish_file", tmp);
	if (rc < 0)
	{
		return;
	}

	rc = xmlTextWriterStartElement(writer, BAD_CAST "questions");
        if (rc < 0) 
        {
            return;
        }


	impr_sql_question_get_list (report_tp.tpid, getuid (), generate_question, writer);
	rc = xmlTextWriterEndElement(writer);

	bzero (fio, 128);
	sprintf (fio, "%s %s", user_info.name, user_info.second);
	tmp = ConvertInput (fio, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "fio_info", tmp);
	if (rc < 0)
	{
		return;
	}

	tmp = ConvertInput (user_info.email, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "email", tmp);
	if (rc < 0)
	{
		return;
	}

	if (!read_file ("/etc/improver/client_version.conf", client_version))
	{
		tmp = ConvertInput (client_version + 16, MY_ENCODING);
		rc = xmlTextWriterWriteElement(writer, BAD_CAST "client_version", tmp);
		if (rc < 0)
		{
		    return;
		}
	}

	rc = xmlTextWriterEndDocument(writer);
	if (rc < 0) 
	{
		return;
	}
	xmlFreeTextWriter(writer);
	xmlFree (tmp);
}
